# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Entry point for all unit-tests for TVB Scientific Library.

.. moduleauthor:: Lia Domide <lia.domide@codemart.ro>
.. moduleauthor:: Bogdan Neacsa <bogdan.neacsa@codemart.ro>
"""

from tvb.tests.library import setup_test_console_env
from tvb.basic.profile import TvbProfile

setup_test_console_env()

# Make sure we are in library mode and are not influenced by framework
try:
    import tvb.interfaces

    raise Exception("Found framework in library mode testing. Abort....")
except ImportError:
    pass

import os
from sys import argv
from coverage import coverage


KEY_CONSOLE = 'console'
KEY_COVERAGE = 'coverage'
KEY_XML = 'xml'


def generate_excludes(root_folders):
    """
    Specify excludes for Coverage.
    """
    excludes = []
    for root in root_folders:
        for root, _, files in os.walk(root):
            for file_n in files:
                full_path = os.path.join(root, file_n)
                if (full_path.endswith('__init__.py') or
                            os.path.join('simulator', 'demos') in full_path):
                    excludes.append(full_path)
    return excludes


if __name__ == "__main__":
    # Start all TVB tests (if in Coverage mode)
    if KEY_COVERAGE in argv:
        import tvb.simulator as sim

        SOURCE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(sim.__file__)))
        COVERAGE = coverage(source=["tvb.basic", "tvb.datatypes", "tvb.simulator"],
                            omit=generate_excludes([SOURCE_DIR]), cover_pylib=False, branch=True)
        COVERAGE.start()
        ## This needs to be executed before any TVB import.

import unittest
import datetime
from tvb.tests.library.basic import basic_test_main
from tvb.tests.library.datatypes import datatypes_test_main
from tvb.tests.library.simulator import simulator_test_main
from tvb.tests.library.xml_runner import XMLTestRunner


def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(basic_test_main.suite())
    test_suite.addTest(datatypes_test_main.suite())
    test_suite.addTest(simulator_test_main.suite())
    return test_suite


if __name__ == "__main__":
    # Start all TVB tests
    START_TIME = datetime.datetime.now()

    if KEY_CONSOLE in argv:
        TEST_RUNNER = unittest.TextTestRunner()
        TEST_SUITE = suite()
        TEST_RUNNER.run(TEST_SUITE)
    if KEY_XML in argv:
        XML_STREAM = open(os.path.join(TvbProfile.current.TVB_LOG_FOLDER, "TEST-LIBRARY-RESULTS.xml"), "w")
        OUT_STREAM = open(os.path.join(TvbProfile.current.TVB_LOG_FOLDER, "TEST-LIBRARY.out"), "w")
        TEST_RUNNER = XMLTestRunner(XML_STREAM, OUT_STREAM)
        TEST_SUITE = suite()
        TEST_RUNNER.run(TEST_SUITE)
        XML_STREAM.close()
        OUT_STREAM.close()

    print('It run tests for %d sec.' % (datetime.datetime.now() - START_TIME).seconds)

    if KEY_COVERAGE in argv:
        COVERAGE.stop()
        COVERAGE.xml_report(outfile=os.path.join(TvbProfile.current.TVB_LOG_FOLDER, 'coverage_library.xml'))
        # COVERAGE.html_report(directory=os.path.join(TvbProfile.current.TVB_LOG_FOLDER, 'test_coverage_html'))



# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#
"""
Created on Mar 20, 2013

.. moduleauthor:: Bogdan Neacsa <bogdan.neacsa@codemart.ro>
"""
if __name__ == "__main__":
    from tvb.tests.library import setup_test_console_env
    setup_test_console_env()

import numpy
import unittest

from tvb.datatypes import spectral, time_series
from tvb.tests.library.base_testcase import BaseTestCase
        
class SpectralTest(BaseTestCase):
    """
    Tests the defaults for `tvb.datatypes.spectral` module.
    """
    
    def test_fourierspectrum(self):
        data = numpy.random.random((10, 10))
        ts = time_series.TimeSeries(data=data)
        dt = spectral.FourierSpectrum(source=ts,
                                      segment_length=100)
        dt.configure()
        summary_info = dt.summary_info
        self.assertEqual(summary_info['Frequency step'], 0.01)     
        self.assertEqual(summary_info['Maximum frequency'], 0.5)  
        self.assertEqual(summary_info['Segment length'], 100)  
        self.assertEqual(summary_info['Windowing function'], '')  
        self.assertEqual(summary_info['Source'], '')  
        self.assertEqual(summary_info['Spectral type'], 'FourierSpectrum')  
        self.assertTrue(dt.aggregation_functions is None)
        self.assertEqual(dt.normalised_average_power.shape, (0, ))
        self.assertEqual(dt.segment_length, 100.0)
        self.assertEqual(dt.shape, (0, ))
        self.assertTrue(dt.source is not None)
        self.assertEqual(dt.windowing_function, '')
        
        
    def test_waveletcoefficients(self):
        data = numpy.random.random((10, 10))
        ts = time_series.TimeSeries(data=data)
        dt = spectral.WaveletCoefficients(source=ts,
                                          mother='morlet',
                                          sample_period=7.8125,
                                          frequencies=[0.008, 0.028, 0.048, 0.068],
                                          normalisation="energy",
                                          q_ratio=5.0,
                                          array_data=numpy.random.random((10, 10)),)
        dt.configure()
        summary_info = dt.summary_info
        self.assertEqual(summary_info['Maximum frequency'], 0.068)  
        self.assertEqual(summary_info['Minimum frequency'], 0.008)  
        self.assertEqual(summary_info['Normalisation'], 'energy')  
        self.assertEqual(summary_info['Number of scales'], 4)
        self.assertEqual(summary_info['Q-ratio'], 5.0)  
        self.assertEqual(summary_info['Sample period'], 7.8125) 
        self.assertEqual(summary_info['Spectral type'], 'WaveletCoefficients')  
        self.assertEqual(summary_info['Wavelet type'], 'morlet')     
        self.assertEqual(dt.q_ratio, 5.0)
        self.assertEqual(dt.sample_period, 7.8125)
        self.assertEqual(dt.shape, (10, 10))
        self.assertTrue(dt.source is not None)
        
        
    def test_coherencespectrum(self):
        data = numpy.random.random((10, 10))
        ts = time_series.TimeSeries(data=data)
        dt = spectral.CoherenceSpectrum(source=ts,
                                        nfft = 4,
                                        array_data = numpy.random.random((10, 10)),
                                        frequency = numpy.random.random((10,)))
        summary_info = dt.summary_info 
        self.assertEqual(summary_info['Number of frequencies'], 10)
        self.assertEqual(summary_info['Spectral type'], 'CoherenceSpectrum')
        self.assertEqual(summary_info['FFT length (time-points)'], 4)
        self.assertEqual(summary_info['Source'], '')
        self.assertEqual(dt.nfft, 4)
        self.assertEqual(dt.shape, (10, 10))
        self.assertTrue(dt.source is not None)
        
        
    def test_complexcoherence(self):
        data = numpy.random.random((10, 10))
        ts = time_series.TimeSeries(data=data)
        dt = spectral.ComplexCoherenceSpectrum(source=ts,
                                               array_data = numpy.random.random((10, 10)),
                                               cross_spectrum = numpy.random.random((10, 10)),
                                               epoch_length = 10,
                                               segment_length = 5)
        summary_info = dt.summary_info
        self.assertEqual(summary_info['Frequency step'], 0.2)     
        self.assertEqual(summary_info['Maximum frequency'], 0.5)  
        self.assertEqual(summary_info['Source'], '')  
        self.assertEqual(summary_info['Spectral type'], 'ComplexCoherenceSpectrum')     
        self.assertTrue(dt.aggregation_functions is None)
        self.assertEqual(dt.epoch_length, 10)
        self.assertEqual(dt.segment_length, 5)
        self.assertEqual(dt.shape, (10, 10))
        self.assertTrue(dt.source is not None)
        self.assertEqual(dt.windowing_function, '')
        
        
def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(SpectralTest))
    return test_suite


if __name__ == "__main__":
    #So you can run tests from this package individually.
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = suite()
    TEST_RUNNER.run(TEST_SUITE) 


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Gather the tests of the simulator module

.. moduleauthor:: Paula Sanz Leon <sanzleon.paula@gmail.com>
"""

# NOTE: for the moment test cases are not relevant (except for the on running all the simulations). 
# They are more like placeholders, but we definitely need to add more exhaustive tests.

if __name__ == "__main__":
    from tvb.tests.library import setup_test_console_env
    setup_test_console_env()


import unittest
from tvb.tests.library.simulator import common_test
from tvb.tests.library.simulator import coupling_test
from tvb.tests.library.simulator import integrators_test
from tvb.tests.library.simulator import models_test
from tvb.tests.library.simulator import monitors_test
from tvb.tests.library.simulator import noise_test
from tvb.tests.library.simulator import simulator_test
from tvb.tests.library.simulator import region_boundaries_test
from tvb.tests.library.simulator import history_test


def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(common_test.suite())
    test_suite.addTest(coupling_test.suite())
    test_suite.addTest(integrators_test.suite())
    test_suite.addTest(history_test.suite())
    test_suite.addTest(models_test.suite())
    test_suite.addTest(monitors_test.suite())
    test_suite.addTest(noise_test.suite())
    test_suite.addTest(region_boundaries_test.suite())
    test_suite.addTest(simulator_test.suite())

    return test_suite


if __name__ == "__main__":
    #So you can run tests from this package individually.
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = suite()
    TEST_RUNNER.run(TEST_SUITE)


# -*- coding: utf-8 -*-
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Small script for converting Brainstorm sensor files for our default dataset
to the simple ASCII format used by TVB (and other software).

NB: Brainstorm uses meters, TVB uses millimeters.

.. moduleauthor:: Marmaduke Woodman <marmaduke.woodman@univ-amu.fr>

"""

import numpy
import scipy.io


def get_field_array(mat_group, n=3, dtype=numpy.float64):
    return numpy.array([
        l.flat[:n] if l.size else numpy.zeros((n, ), dtype)
        for l in mat_group[0]
    ], dtype=dtype)

def convert_brainstorm_to_tvb(tvb_data_path, chan_paths):
    """
    Convert given set of channels from Brainstorm to TVB formats.

    """

    bst_path = tvb_data_path + 'brainstorm/data/TVB-Subject/'
    for sens_type, sens_path in chan_paths.iteritems():
        # only MEG channels require orientation information
        use_ori = sens_type in ('meg', )
        # read from MAT file necessary fields
        mat = scipy.io.loadmat(bst_path + sens_path)
        name = [l[0] for l in mat['Channel']['Name'][0]]
        loc = get_field_array(mat['Channel']['Loc'])
        if use_ori:
            ori = get_field_array(mat['Channel']['Orient'])
        # bst uses m, we use mm
        loc *= 1e3
        # write out to text format
        out_fname = '%s/sensors/%s-brainstorm-%d.txt'
        out_fname %= tvb_data_path, sens_type, len(name)
        with open(out_fname, 'w') as fd:
            if use_ori: # MEG
                for n, (x, y, z), (ox, oy, oz) in zip(name, loc, ori):
                    line = '\t'.join(['%s']+['%f']*6) + '\n'
                    line %= n, x, y, z, ox, oy, oz
                    fd.write(line)
            else: # sEEG, EEG
                for n, (x, y, z) in zip(name, loc):
                    line = '\t'.join(['%s']+['%f']*3) + '\n'
                    line %= n, x, y, z
                    fd.write(line)


if __name__ == '__main__':
    import tvb_data
    convert_brainstorm_to_tvb(tvb_data.__path__,
        chan_paths={
            'eeg': 'EEG_channels/channel.mat',
            'meg': 'MEGchannels/channel_4d_acc1.mat',
            'seeg': 'seeg_channels/channel.mat',
        })


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
.. moduleauthor:: Bogdan Neacsa <bogdan.neacsa@codemart.ro>
"""
from tvb.datatypes.cortex import Cortex
from tvb.datatypes.local_connectivity import LocalConnectivity
from tvb.datatypes.region_mapping import RegionMapping

if __name__ == "__main__":
    from tvb.tests.library import setup_test_console_env
    setup_test_console_env()

import unittest
import sys
import numpy
from tvb.datatypes import surfaces
from tvb.tests.library.base_testcase import BaseTestCase



class SurfacesTest(BaseTestCase):
    """
    Tests the defaults for `tvb.datatypes.surfaces` module.
    """

    def test_surface(self):
        dt = surfaces.Surface()
        dt.vertices = numpy.array(range(30)).reshape(10, 3).astype(numpy.float64)
        dt.triangles = numpy.array(range(9)).reshape(3, 3)
        dt.configure()
        summary_info = dt.summary_info
        self.assertEqual(summary_info['Number of edges'], 9)
        self.assertEqual(summary_info['Number of triangles'], 3)
        self.assertEqual(summary_info['Number of vertices'], 10)
        self.assertEqual(summary_info['Surface type'], 'Surface')
        self.assertEqual(len(dt.vertex_neighbours), 10)
        self.assertTrue(isinstance(dt.vertex_neighbours[0], frozenset))
        self.assertEqual(len(dt.vertex_triangles), 10)
        self.assertTrue(isinstance(dt.vertex_triangles[0], frozenset))
        self.assertEqual(len(dt.nth_ring(0)), 0)
        self.assertEqual(dt.triangle_areas.shape, (3, 1))
        self.assertEqual(dt.triangle_angles.shape, (3, 3))
        self.assertEqual(len(dt.edges), 9)
        self.assertEqual(len(dt.edge_triangles), 9)
        self.assertNotEqual([], dt.validate_topology_for_simulations().warnings)
        self.assertEqual(dt.get_data_shape('vertices'), (10, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (10, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (3, 3))


    def test_cortical_surface(self):
        dt = surfaces.CorticalSurface(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.CorticalSurface))
        dt.configure()
        summary_info = dt.summary_info
        self.assertEqual(summary_info['Number of edges'], 49140)
        self.assertEqual(summary_info['Number of triangles'], 32760)
        self.assertEqual(summary_info['Number of vertices'], 16384)
        self.assertEqual(dt.surface_type, surfaces.CORTICAL)
        self.assertEqual(len(dt.vertex_neighbours), 16384)
        self.assertTrue(isinstance(dt.vertex_neighbours[0], frozenset))
        self.assertEqual(len(dt.vertex_triangles), 16384)
        self.assertTrue(isinstance(dt.vertex_triangles[0], frozenset))
        self.assertEqual(len(dt.nth_ring(0)), 17)
        self.assertEqual(dt.triangle_areas.shape, (32760, 1))
        self.assertEqual(dt.triangle_angles.shape, (32760, 3))
        self.assertEqual(len(dt.edges), 49140)
        self.assertTrue(abs(dt.edge_length_mean - 3.97605292887) < 0.00000001)
        self.assertTrue(abs(dt.edge_length_min - 0.6638) < 0.0001)
        self.assertTrue(abs(dt.edge_length_max - 7.7567) < 0.0001)
        self.assertEqual(len(dt.edge_triangles), 49140)
        self.assertEqual([], dt.validate_topology_for_simulations().warnings)
        self.assertEqual(dt.get_data_shape('vertices'), (16384, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (16384, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (32760, 3))
        topologicals = dt.compute_topological_constants()
        self.assertEqual(4, topologicals[0])
        self.assertTrue(all([a.size == 0 for a in topologicals[1:]]))


    def test_cortical_topology_pyramid(self):
        dt = surfaces.Surface()
        dt.vertices = numpy.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]]).astype(numpy.float64)
        dt.triangles = numpy.array([[0, 2, 1], [0, 1, 3], [0, 3, 2], [1, 2, 3]])
        dt.configure()

        euler, isolated, pinched_off, holes = dt.compute_topological_constants()
        self.assertEqual(2, euler)
        self.assertEqual(0, isolated.size)
        self.assertEqual(0, pinched_off.size)
        self.assertEqual(0, holes.size)


    def test_cortical_topology_isolated_vertex(self):
        dt = surfaces.Surface()
        dt.vertices = numpy.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1], [0, 0, 2]]).astype(numpy.float64)
        dt.triangles = numpy.array([[0, 2, 1], [0, 1, 3], [0, 3, 2], [1, 2, 3]])
        dt.configure()

        euler, isolated, pinched_off, holes = dt.compute_topological_constants()
        self.assertEqual(3, euler)
        self.assertEqual(1, isolated.size)
        self.assertEqual(0, pinched_off.size)
        self.assertEqual(0, holes.size)


    def test_cortical_topology_pinched(self):
        dt = surfaces.Surface()
        dt.vertices = numpy.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]]).astype(numpy.float64)
        dt.triangles = numpy.array([[0, 2, 1], [0, 1, 3], [0, 3, 2], [1, 2, 3], [1, 2, 3]])
        dt.configure()

        euler, isolated, pinched_off, holes = dt.compute_topological_constants()
        self.assertEqual(3, euler)
        self.assertEqual(0, isolated.size)
        self.assertEqual(3, pinched_off.size)
        self.assertEqual(0, holes.size)


    def test_cortical_topology_hole(self):
        dt = surfaces.Surface()
        dt.vertices = numpy.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]]).astype(numpy.float64)
        dt.triangles = numpy.array([[0, 2, 1], [0, 1, 3], [0, 3, 2]])
        dt.configure()

        euler, isolated, pinched_off, holes = dt.compute_topological_constants()
        self.assertEqual(1, euler)
        self.assertEqual(3, isolated.size)
        self.assertEqual(0, pinched_off.size)
        self.assertEqual(3, holes.size)


    def test_skinair(self):
        dt = surfaces.SkinAir(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.SkinAir))
        self.assertEqual(dt.get_data_shape('vertices'), (4096, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (4096, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (8188, 3))


    def test_brainskull(self):
        dt = surfaces.BrainSkull(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.BrainSkull))
        self.assertEqual(dt.get_data_shape('vertices'), (4096, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (4096, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (8188, 3))


    def test_skullskin(self):
        dt = surfaces.SkullSkin(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.SkullSkin))
        self.assertEqual(dt.get_data_shape('vertices'), (4096, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (4096, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (8188, 3))


    def test_eegcap(self):
        dt = surfaces.EEGCap(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.EEGCap))
        self.assertEqual(dt.get_data_shape('vertices'), (1082, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (1082, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (2160, 3))


    def test_facesurface(self):
        dt = surfaces.FaceSurface(load_default=True)
        self.assertTrue(isinstance(dt, surfaces.FaceSurface))
        self.assertEqual(dt.get_data_shape('vertices'), (8614, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (0,))
        self.assertEqual(dt.get_data_shape('triangles'), (17224, 3))


    def test_regionmapping(self):
        dt = RegionMapping(load_default=True)
        self.assertTrue(isinstance(dt, RegionMapping))
        self.assertEqual(dt.shape, (16384,))


    def test_localconnectivity_empty(self):
        dt = LocalConnectivity()
        self.assertTrue(dt.surface is None)


    @unittest.skipIf(sys.maxsize <= 2147483647, "Cannot deal with local connectivity on a 32-bit machine.")
    def test_cortexdata(self):

        dt = Cortex(load_default=True)
        self.assertTrue(isinstance(dt, Cortex))
        self.assertTrue(dt.region_mapping is not None)
        ## Initialize Local Connectivity, to avoid long computation time.
        dt.local_connectivity = LocalConnectivity(load_default=True)

        dt.configure()
        summary_info = dt.summary_info
        self.assertTrue(abs(summary_info['Region area, maximum (mm:math:`^2`)'] - 9333.39) < 0.01)
        self.assertTrue(abs(summary_info['Region area, mean (mm:math:`^2`)'] - 3038.51) < 0.01)
        self.assertTrue(abs(summary_info['Region area, minimum (mm:math:`^2`)'] - 540.90) < 0.01)
        self.assertEqual(dt.get_data_shape('vertices'), (16384, 3))
        self.assertEqual(dt.get_data_shape('vertex_normals'), (16384, 3))
        self.assertEqual(dt.get_data_shape('triangles'), (32760, 3))



def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(SurfacesTest))
    return test_suite



if __name__ == "__main__":
    #So you can run tests from this package individually.
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = suite()
    TEST_RUNNER.run(TEST_SUITE) 
    
    


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#
"""
.. moduleauthor:: Calin Pavel <calin.pavel@codemart.ro>
.. moduleauthor:: Lia Domide <lia.domide@codemart.ro>
"""
if __name__ == "__main__":
    from tvb.tests.library import setup_test_console_env
    setup_test_console_env()
    
import unittest
import numpy
import json
from copy import deepcopy
import tvb.datatypes.equations as equations 
import tvb.datatypes.arrays as arrays
import tvb.datatypes.time_series as time_series
from tvb.datatypes.equations import Equation
from tvb.basic.traits.types_basic import MapAsJson
from tvb.basic.traits.types_mapped import MappedType
from tvb.basic.traits import types_basic as basic
from tvb.simulator.models import WilsonCowan, ReducedSetHindmarshRose
from tvb.tests.library.base_testcase import BaseTestCase


class TraitsTest(BaseTestCase):
    """
    Test class for traits.core and traits.base
    """
        
    def test_default_attributes(self):
        """
        Test that default attributes are populated as they are described in Traits.
        """
        model_wc = WilsonCowan()
        self.assertTrue(type(model_wc.c_ii) == numpy.ndarray)
        self.assertEqual(1, len(model_wc.c_ii))
        self.assertTrue(type(model_wc.tau_e) == numpy.ndarray)
        self.assertEqual(1, len(model_wc.tau_e))
        self.assertTrue(isinstance(model_wc._tau_e, str))
        
        
    def test_modifying_attributes(self):
        """ 
        Test that when modifying an instance attributes, they are only visible on that instance.
        """
        eqn_t = equations.Gaussian()
        eqn_t.parameters["midpoint"] = 8.0
        
        eqn_x = equations.Gaussian()
        eqn_x.parameters["amp"] =  -0.0625
        
        self.assertTrue(eqn_t.parameters is not None)
        self.assertEqual(eqn_t.parameters['amp'], 1.0)
        self.assertEqual(eqn_t.parameters['midpoint'], 8.0)
        self.assertTrue(eqn_x.parameters is not None)
        self.assertEqual(eqn_x.parameters['amp'], -0.0625)
        self.assertEqual(eqn_x.parameters['midpoint'], 0.0)
        
        a1 = arrays.FloatArray()
        a1.data = numpy.array([20.0,])
        a2 = deepcopy(a1)
        a2.data = numpy.array([42.0,])
        self.assertNotEqual(a1.data[0], a2.data[0])
        
        model1 = ReducedSetHindmarshRose()
        model1.a = numpy.array([55.0,])
        model1.number_of_modes = 10
        
        model2 = ReducedSetHindmarshRose()
        model2.a = numpy.array([42.0])
        model2.number_of_modes = 15
        self.assertNotEqual(model1.number_of_modes, model2.number_of_modes)
        self.assertNotEqual(model1.a[0], model2.a[0])
      
      
    def test_array_populated(self):
        """
        simple Array test
        """
        arr = arrays.MappedArray(array_data = numpy.array(range(10)))
        arr.configure()
        self.assertEquals(10, arr.array_data.shape[0])  
        self.assertEquals(1, arr.nr_dimensions)
        self.assertEquals(10, arr.length_1d)
        
    def test_linked_attributes(self):
        """
        Test that tvb.core.trait.util produces something.
        """
        class Internal_Class(MappedType):
            """ Dummy persisted class"""
            x = 5
            z = basic.Integer()
            j = basic.JSONType()
            class In_Internal_Class(object):
                """Internal of Internal class"""
                t = numpy.array(range(10))
            @property
            def y(self):
                return self.x
        
        instance =  Internal_Class()   
        self.assertEqual(5, instance.y)
        
        instance.j = {'dict_key': 10}
        self.assertTrue(isinstance(instance._j, str))
        self.assertTrue(isinstance(instance.j, dict))
       
    
    def test_none_complex_attribute(self):
        """
        Test that traited attributes are returned None, 
        when no value is assigned as default to them.
        """
        serie = time_series.TimeSeriesRegion()
        self.assertTrue(serie.connectivity is None) 
        
        
    def test_json_dumps_loads(self):
        """
        Tests class `MapAsJson.MapAsJsonEncoder` loads parameters correctly from JSON.
        """
        input_parameters = {'a':1, 'b':1.0, 'c':'d'}
        test_dict = {'1' : 1, 'a' : {'1' : 'b'}, '2' : {'a' : Equation(parameters=input_parameters)}}
        json_string = json.dumps(test_dict, cls=MapAsJson.MapAsJsonEncoder)
        loaded_dict =  json.loads(json_string, object_hook=MapAsJson.decode_map_as_json)
        eq_parameters = loaded_dict['2']['a']
        self.assertEqual(input_parameters, eq_parameters.parameters, "parameters not loaded properly from json")
        
        
def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(TraitsTest))
    return test_suite


if __name__ == "__main__":
    #So you can run tests from this package individually.
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = suite()
    TEST_RUNNER.run(TEST_SUITE) 
    
    
    
    


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Calculate temporal cross correlation on a TimeSeries datatype and return a 
temporal_correlations.CrossCorrelation dataype.

.. moduleauthor:: Stuart A. Knock <Stuart@tvb.invalid>

"""

import numpy
import tvb.datatypes.time_series as time_series
import tvb.datatypes.temporal_correlations as temporal_correlations
import tvb.basic.traits.core as core
import tvb.basic.traits.util as util
from scipy.signal.signaltools import correlate
from tvb.basic.logger.builder import get_logger

LOG = get_logger(__name__)




class CrossCorrelate(core.Type):
    """
    Compute the node-pairwise cross-correlation of the given input 4D TimeSeries DataType.
    
    Return a CrossCorrelation DataType. It contains the cross-correlation
    sequences for all possible combinations of the nodes.
    
    See: http://www.scipy.org/doc/api_docs/SciPy.signal.signaltools.html#correlate
    """

    time_series = time_series.TimeSeries(
        label="Time Series",
        required=True,
        doc="""The time-series for which the cross correlation sequences are calculated.""")
    
    
    def evaluate(self):
        """
        Cross-correlate two one-dimensional arrays.
        """
        cls_attr_name = self.__class__.__name__ + ".time_series"
        self.time_series.trait["data"].log_debug(owner=cls_attr_name)
        
        #(tpts, nodes, nodes, state-variables, modes)
        result_shape = self.result_shape(self.time_series.data.shape)
        LOG.info("result shape will be: %s" % str(result_shape))
        
        result = numpy.zeros(result_shape)
        
        #TODO: For region level, 4s, 2000Hz, this takes ~3hours...(which makes node_coherence seem positively speedy...)
        # Probably best to add a keyword for offsets, so we just compute +- some "small" range...
        # One inter-node correlation, across offsets, for each state-var & mode.
        for mode in range(result_shape[4]):
            for var in range(result_shape[3]):
                data = self.time_series.data[:, var, :, mode]
                data = data - data.mean(axis=0)[numpy.newaxis, :]
                #TODO: Work out a way around the 4 level loop:
                for n1 in range(result_shape[1]):
                    for n2 in range(result_shape[2]):
                        result[:, n1, n2, var, mode] = correlate(data[:, n1], data[:, n2], mode="same")
        
        util.log_debug_array(LOG, result, "result")
        
        offset = (self.time_series.sample_period *
                  numpy.arange(-numpy.floor(result_shape[0] / 2.0), numpy.ceil(result_shape[0] / 2.0)))

        cross_corr = temporal_correlations.CrossCorrelation(
            source=self.time_series,
            array_data=result,
            time=offset,
            use_storage=False)
        
        return cross_corr
    
    
    def result_shape(self, input_shape):
        """Returns the shape of the main result of ...."""
        result_shape = (input_shape[0], input_shape[2], input_shape[2], input_shape[1], input_shape[3])
        return result_shape
    
    
    def result_size(self, input_shape):
        """
        Returns the storage size in Bytes of the main result of .
        """
        result_size = numpy.sum(map(numpy.prod, self.result_shape(input_shape))) * 8.0  # Bytes
        return result_size






# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Module to handle framework specific methods related to noise sources.


.. moduleauthor:: Marmaduke Woodman <marmaduke.woodman@univ-amu.fr>

"""

import numpy.random

import tvb.basic.traits.parameters_factory as parameters_factory
from tvb.datatypes import equations
from tvb.simulator.noise import Noise

KEY_NOISE = "noise"
PARAMS_NOISE = (KEY_NOISE + '_parameters_')[:-1]
KEY_RANDOM_STREAM = "random_stream"
PARAMS_RANDOM_STREAM = (KEY_RANDOM_STREAM + '_parameters_')[:-1]
KEY_EQUATION = "b"
PARAMS_EQUATION = "b_parameters"


def build_noise(parent_parameters):
    """
    Build Noise entity from dictionary of parameters.
    :param parent_parameters: dictionary of parameters for the entity having Noise as attribute. \
                              The dictionary is after UI form-submit and framework pre-process.
    :return: Noise entity.

    """

    if KEY_NOISE not in parent_parameters:
        return None

    available_noise = parameters_factory.get_traited_subclasses(Noise)
    if 'Noise' not in available_noise:
        available_noise['Noise'] = Noise

    selected_noise = parent_parameters[KEY_NOISE]
    noise_params = parent_parameters[PARAMS_NOISE]
    del parent_parameters[KEY_NOISE]
    del parent_parameters[PARAMS_NOISE]

    if PARAMS_RANDOM_STREAM in noise_params:
        stream_params = noise_params[PARAMS_RANDOM_STREAM]
        random_stream = numpy.random.RandomState(seed=stream_params['init_seed'])
        del noise_params[PARAMS_RANDOM_STREAM]
        del noise_params[KEY_RANDOM_STREAM]
        noise_params[KEY_RANDOM_STREAM] = random_stream

    if PARAMS_EQUATION in noise_params:
        available_equations = parameters_factory.get_traited_subclasses(equations.Equation)
        eq_parameters = noise_params[PARAMS_EQUATION]["parameters"]
        equation = noise_params[KEY_EQUATION]
        equation = available_equations[equation](parameters=eq_parameters)
        del noise_params[PARAMS_EQUATION]
        del noise_params[KEY_EQUATION]
        noise_params[KEY_EQUATION] = equation

    noise_entity = available_noise[str(selected_noise)](**noise_params)
    parent_parameters[KEY_NOISE] = noise_entity
    return noise_entity





# -*- coding: utf-8 -*-
#
#
# TheVirtualBrain-Scientific Package. This package holds all simulators, and
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#


"""
This is the tests folder for the entire TVB project.
"""

import unittest
from pkgutil import extend_path

__path__ = extend_path(__path__, __name__)


def run_all():
    run_framework()
    run_library()


def run_framework():
    try:
        from tvb.tests.framework.main_tests import suite as fwk_suite
        TEST_RUNNER = unittest.TextTestRunner()
        TEST_SUITE = fwk_suite()
        TEST_RUNNER.run(TEST_SUITE)
    except ImportError:
        # ignore this when framework does not exist
        pass


def run_library():
    from tvb.tests.library.main_tests import suite as lib_suite
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = lib_suite()
    TEST_RUNNER.run(TEST_SUITE)


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
All the little functions that make life nicer in the Traits package.

.. moduleauthor:: Mihai Andrei <mihai.andrei@codemart.ro>
.. moduleauthor:: Lia Domide <lia.domide@codemart.ro>
.. moduleauthor:: marmaduke <duke@eml.cc>
"""

import numpy
import collections
import inspect
from tvb.basic.profile import TvbProfile


# returns true if key is, by convention, public
ispublic = lambda key: key[0] is not '_'


def str_class_name(thing, short_form=False):
    """
    A helper function that tries to generate an informative name for its
    argument: when passed a class, return its name, when passed an object
    return a string representation of that value.
    """
    # if thing is a class, it has attribute __name__
    if hasattr(thing, '__name__'):
        cls = thing
        if short_form:
            return cls.__name__
        return cls.__module__ + '.' + cls.__name__
    else:
        # otherwise, it's an object and we return its __str__
        return str(thing)



def get(obj, key, default=None):
    """
    get() is a general function allowing us to ignore whether we are
    getting from a dictionary or object. If obj is a dictionary, we
    return the value corresponding to key, otherwise we return the
    attribute on obj corresponding to key. In both cases, if key
    does not exist, default is returned.
    """
    if type(obj) is dict:
        return obj.get(key, default)
    else:
        return getattr(obj, key) if hasattr(obj, key) else default



def log_debug_array(log, array, array_name, owner=""):
    """
    Simple access to debugging info on an array.
    """
    if TvbProfile.current.TRAITS_CONFIGURATION.use_storage:
        return
        # Hide this logs in web-mode, with storage, because we have multiple storage exceptions

    if owner != "":
        name = ".".join((owner, array_name))
    else:
        name = array_name

    if array is not None and hasattr(array, 'shape'):
        shape = str(array.shape)
        dtype = str(array.dtype)
        has_nan = str(numpy.isnan(array).any())
        array_max = str(array.max())
        array_min = str(array.min())
        log.debug("%s shape: %s" % (name, shape))
        log.debug("%s dtype: %s" % (name, dtype))
        log.debug("%s has NaN: %s" % (name, has_nan))
        log.debug("%s maximum: %s" % (name, array_max))
        log.debug("%s minimum: %s" % (name, array_min))
    else:
        log.debug("%s is None or not Array" % name)



Args = collections.namedtuple('Args', 'pos kwd')



class TypeRegister(list):
    """
    TypeRegister is a smart list that can be queried to obtain selections of the
    classes inheriting from Traits classes.
    """


    def subclasses(self, obj, avoid_subclasses=False):
        """
        The subclasses method takes a class (or given instance object, will use
        the class of the instance), and returns a list of all options known to
        this TypeRegister that are direct subclasses of the class or have the
        class in their base class list.
        :param obj: Class or instance
        :param avoid_subclasses: When specified, subclasses are not retrieved, only current class.
        """

        cls = obj if inspect.isclass(obj) else obj.__class__

        if avoid_subclasses:
            return [cls]

        if hasattr(cls, '_base_classes'):
            bases = cls._base_classes
        else:
            bases = []

        sublcasses = [opt for opt in self if ((issubclass(opt, cls) or cls in opt.__bases__)
                                              and not inspect.isabstract(opt) and opt.__name__ not in bases)]
        return sublcasses


def multiline_math_directives_to_matjax(doc):
    """
    Looks for multi-line sphinx math directives in the given rst string
    It converts them in html text that will be interpreted by mathjax
    The parsing is simplistic, not a rst parser.
    Wraps .. math :: body in \[\begin{split}\end{split}\]
    """

    # doc = text | math
    BEGIN = r'\[\begin{split}'
    END = r'\end{split}\]'

    in_math = False  # 2 state parser
    out_lines = []
    indent = ''

    for line in doc.splitlines():
        if not in_math:
            # math = indent directive math_body
            indent, sep, _ = line.partition('.. math::')
            if sep:
                out_lines.append(BEGIN)
                in_math = True
            else:
                out_lines.append(line)
        else:
            # math body is at least 1 space more indented than the directive, but we tolerate empty lines
            if line.startswith(indent + ' ') or line.strip() == '':
                out_lines.append(line)
            else:
                # this line is not properly indented, math block is over
                out_lines.append(END)
                out_lines.append(line)
                in_math = False

    if in_math:
        # close math tag
        out_lines.append(END)

    return '\n'.join(out_lines)


# -*- coding: utf-8 -*-
#
#
# TheVirtualBrain-Framework Package. This package holds all Data Management, and
# Web-UI helpful to run brain-simulations. To use it, you also need do download
# TheVirtualBrain-Scientific Package (for simulators). See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
.. moduleauthor:: Lia Domide <lia.domide@codemart.ro>
"""

import numpy
from tvb.basic.traits import types_basic
from tvb.basic.traits.types_mapped import MappedType, Array
from tvb.datatypes import connectivity


ANNOTATION_DTYPE = numpy.dtype([('id', 'i'),
                                ('parent_id', 'i'),
                                ('parent_left', 'i'),
                                ('parent_right', 'i'),
                                ('relation', 'S16'),
                                ('label', 'S128'),
                                ('definition', 'S1024'),
                                ('synonym', 'S2048'),
                                ('uri', 'S248'),
                                ('synonym_tvb_left', 'i'),
                                ('synonym_tvb_right', 'i')
                                ])

ICON_TVB = "/static/style/nodes/nodeRoot.png"
ICON_FOLDER = "/static/style/nodes/nodeFolder.png"
NODE_ID_TVB = "node_tvb_"
NODE_ID_TVB_ROOT = "node_tvb_root_"
NODE_ID_BRCO = "node_brco_"


class AnnotationTerm(object):
    """
    One single annotation node (in the tree of annotations / region)
    """


    def __init__(self, id, parent, parent_left, parent_right, relation, label, definition=None, synonym=None, uri=None,
                 tvb_left=None, tvb_right=None):
        self.id = id
        self.parent_id = parent
        self.parent_left = parent_left
        self.parent_right = parent_right
        self.relation = relation
        self.label = label
        self.definition = definition
        self.synonym = synonym
        self.uri = uri
        self.synonym_tvb_left = tvb_left or -1
        self.synonym_tvb_right = tvb_right or -1
        self.children = []


    def add_child(self, annotation_child):
        self.children.append(annotation_child)


    def to_tuple(self):
        return self.id, self.parent_id, self.parent_left, self.parent_right, \
               self.relation, self.label, self.definition, self.synonym, self.uri, \
               self.synonym_tvb_left, self.synonym_tvb_right


    def to_json(self, is_right_hemisphere=False, activation_patterns=None):

        children = []
        for child in self.children:
            children.append(child.to_json(is_right_hemisphere, activation_patterns))
        title = "URI: " + self.uri + "\n\nLabel: " + self.label + "\n\nDefinition: " + self.definition + \
                "\n\nSynonyms: " + self.synonym.replace("|", "\n")
        if activation_patterns is not None and str(self.id) in activation_patterns:
            connected_regions = activation_patterns[str(self.id)]
            title += "\n\nTVB " + str(len(connected_regions)) + " connected regions: " + str(connected_regions)

        if self.synonym_tvb_right >= 0 and self.synonym_tvb_left >= 0:
            # When TVB regions display differently
            synonym_id = self.synonym_tvb_right if is_right_hemisphere else self.synonym_tvb_left
            short_tvb_name = self.uri.split('#')[1]
            title = str(synonym_id) + " - " + short_tvb_name + "\n\n" + title
            return dict(data=dict(icon=ICON_TVB, title=short_tvb_name),
                        attr=dict(id=NODE_ID_TVB + str(synonym_id), title=title),
                        state="close", children=children)

        return dict(data=dict(icon=ICON_FOLDER, title=self.label.capitalize()),
                    attr=dict(id=NODE_ID_BRCO + str(self.id), title=title),
                    state="close", children=children)



class AnnotationArray(Array):
    """
    Holds a flatten form for the annotations for a full connectivity.
    Each region in the connectivity can have None, or a tree of AnnotationTerms
    To be stored in a compound DS in H5.
    """

    dtype = types_basic.DType(default=ANNOTATION_DTYPE)

    stored_metadata = [MappedType.METADATA_ARRAY_SHAPE]



class ConnectivityAnnotations(MappedType):
    """
    Ontology annotations for a Connectivity.
    """

    connectivity = connectivity.Connectivity

    region_annotations = AnnotationArray(
        default=numpy.array([], dtype=ANNOTATION_DTYPE),
        label="Region Annotations",
        doc="""Flat tree of annotations for every connectivity region.""")


    def set_annotations(self, annotation_terms):
        annotations = [ann.to_tuple() for ann in annotation_terms]
        annotations = numpy.array(annotations, dtype=ANNOTATION_DTYPE)
        self.region_annotations = annotations


    def _find_summary_info(self):
        """
        Gather interesting summary information from an instance of this dataType.
        """
        summary = {"Connectivity": self.connectivity.display_name}
        summary.update(self.get_info_about_array('region_annotations', [self.METADATA_ARRAY_SHAPE]))
        return summary


    def get_activation_patterns(self):
        """
        Group Annotation terms by URI.
        :return: map {brco_id: list of TVB regions IDs in which the same term is being subclass}
        """

        map_by_uri = {}
        for ann in self.region_annotations:
            ann_uri = ann[8]
            left, right = str(ann[2]), str(ann[3])
            if ann_uri not in map_by_uri:
                map_by_uri[ann_uri] = [left, right]
            else:
                if left not in map_by_uri[ann_uri]:
                    map_by_uri[ann_uri].append(left)
                if right not in map_by_uri[ann_uri]:
                    map_by_uri[ann_uri].append(right)

        map_by_brco_id = {}
        for ann in self.region_annotations:
            ann_uri = ann[8]
            ann_id = ann[0]
            map_by_brco_id[str(ann_id)] = map_by_uri[ann_uri]

        return map_by_brco_id


    def get_activation_pattern_labels(self):
        """
        :return: map {brco_id: list of TVB regions LABELS in which the same BRCO term is being subclass}
        """
        map_with_ids = self.get_activation_patterns()
        map_with_labels = dict()

        for ann_id, activated_ids in map_with_ids.iteritems():
            map_with_labels[ann_id] = []
            for string_idx in activated_ids:
                int_idx = int(string_idx)
                conn_label = self.connectivity.region_labels[int_idx]
                map_with_labels[ann_id].append(conn_label)

        return map_with_labels


    def tree_json(self):
        """
        :return: JSON to be rendered in a Tree of entities
        """
        annotations_map = dict()
        regions_map = dict()
        for i in range(self.connectivity.number_of_regions):
            regions_map[i] = []

        for ann in self.region_annotations:
            ann_obj = AnnotationTerm(ann[0], ann[1], ann[2], ann[3], ann[4], ann[5],
                                     ann[6], ann[7], ann[8], ann[9], ann[10])
            annotations_map[ann_obj.id] = ann_obj
            if ann_obj.parent_id < 0:
                # Root directly under a TVB region node
                regions_map[ann_obj.parent_left].append(ann_obj)
                regions_map[ann_obj.parent_right].append(ann_obj)
            elif ann_obj.parent_id in annotations_map:
                annotations_map[ann_obj.parent_id].add_child(ann_obj)
            else:
                self.logger.warn("Order of processing invalid parent %s child %s" % (ann_obj.parent_id, ann_obj.id))

        left_nodes, right_nodes = [], []
        activation_patterns = self.get_activation_pattern_labels()
        for region_idx, annotations_list in regions_map.iteritems():
            if_right_hemisphere = self.connectivity.is_right_hemisphere(region_idx)
            childred_json = []
            for ann_term in annotations_list:
                childred_json.append(ann_term.to_json(if_right_hemisphere, activation_patterns))
            # This node is built for every TVB region
            child_json = dict(data=dict(icon=ICON_TVB,
                                        title=self.connectivity.region_labels[region_idx]),
                              attr=dict(id=NODE_ID_TVB_ROOT + str(region_idx),
                                        title=str(region_idx) + " - " + self.connectivity.region_labels[region_idx]),
                              state="close", children=childred_json)
            if if_right_hemisphere:
                right_nodes.append(child_json)
            else:
                left_nodes.append(child_json)

        # Group everything under a single root
        left_root = dict(data=dict(title="Left Hemisphere", icon=ICON_FOLDER),
                         state="open", children=left_nodes)
        right_root = dict(data=dict(title="Right Hemisphere", icon=ICON_FOLDER),
                          state="open", children=right_nodes)
        root_root = dict(data=dict(title=self.display_name, icon=ICON_FOLDER),
                         state="open", children=[left_root, right_root])
        return root_root



# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""

The Spectral datatypes. This brings together the scientific and framework
methods that are associated with the Spectral datatypes.

.. moduleauthor:: Stuart A. Knock <Stuart@tvb.invalid>

"""
import json
import numpy
from tvb.basic.logger.builder import get_logger
from tvb.basic.traits import util, core, types_basic as basic
from tvb.datatypes import arrays, time_series

LOG = get_logger(__name__)


class FourierSpectrum(arrays.MappedArray):
    """
    Result of a Fourier  Analysis.
    """
    # Overwrite attribute from superclass
    array_data = arrays.ComplexArray(file_storage=core.FILE_STORAGE_EXPAND)

    source = time_series.TimeSeries(
        label="Source time-series",
        doc="Links to the time-series on which the FFT is applied.")

    segment_length = basic.Float(
        label="Segment length",
        doc="""The timeseries was segmented into equally sized blocks
            (overlapping if necessary), prior to the application of the FFT.
            The segement length determines the frequency resolution of the
            resulting spectra.""")

    windowing_function = basic.String(
        label="Windowing function",
        doc="""The windowing function applied to each time segment prior to
            application of the FFT.""")

    amplitude = arrays.FloatArray(
        label="Amplitude",
        file_storage=core.FILE_STORAGE_EXPAND)

    phase = arrays.FloatArray(
        label="Phase",
        file_storage=core.FILE_STORAGE_EXPAND)

    power = arrays.FloatArray(
        label="Power",
        file_storage=core.FILE_STORAGE_EXPAND)

    average_power = arrays.FloatArray(
        label="Average Power",
        file_storage=core.FILE_STORAGE_EXPAND)

    normalised_average_power = arrays.FloatArray(
        label="Normalised Power",
        file_storage=core.FILE_STORAGE_EXPAND)

    _frequency = None
    _freq_step = None
    _max_freq = None

    __generate_table__ = True

    def configure(self):
        """After populating few fields, compute the rest of the fields"""
        # Do not call super, because that accesses data not-chunked
        self.nr_dimensions = len(self.read_data_shape())
        for i in range(self.nr_dimensions):
            setattr(self, 'length_%dd' % (i + 1), int(self.read_data_shape()[i]))

        if self.trait.use_storage is False and sum(self.get_data_shape('array_data')) != 0:
            if self.amplitude.size == 0:
                self.compute_amplitude()
            if self.phase.size == 0:
                self.compute_phase()
            if self.power.size == 0:
                self.compute_power()
            if self.average_power.size == 0:
                self.compute_average_power()
            if self.normalised_average_power.size == 0:
                self.compute_normalised_average_power()

    def write_data_slice(self, partial_result):
        """
        Append chunk.
        """
        # self.store_data_chunk('array_data', partial_result, grow_dimension=2, close_file=False)

        self.store_data_chunk('array_data', partial_result.array_data, grow_dimension=2, close_file=False)

        partial_result.compute_amplitude()
        self.store_data_chunk('amplitude', partial_result.amplitude, grow_dimension=2, close_file=False)

        partial_result.compute_phase()
        self.store_data_chunk('phase', partial_result.phase, grow_dimension=2, close_file=False)

        partial_result.compute_power()
        self.store_data_chunk('power', partial_result.power, grow_dimension=2, close_file=False)

        partial_result.compute_average_power()
        self.store_data_chunk('average_power', partial_result.average_power, grow_dimension=2, close_file=False)

        partial_result.compute_normalised_average_power()
        self.store_data_chunk('normalised_average_power', partial_result.normalised_average_power,
                              grow_dimension=2, close_file=False)

    def _find_summary_info(self):
        """
        Gather scientifically interesting summary information from an instance of this datatype.
        """
        summary = {"Spectral type": self.__class__.__name__,
                   "Source": self.source.title,
                   "Segment length": self.segment_length,
                   "Windowing function": self.windowing_function,
                   "Frequency step": self.freq_step,
                   "Maximum frequency": self.max_freq}
        return summary

    @property
    def freq_step(self):
        """ Frequency step size of the complex Fourier spectrum."""
        if self._freq_step is None:
            self._freq_step = 1.0 / self.segment_length
            msg = "%s: Frequency step size is %s"
            LOG.debug(msg % (str(self), str(self._freq_step)))
        return self._freq_step

    @property
    def max_freq(self):
        """ Amplitude of the complex Fourier spectrum."""
        if self._max_freq is None:
            self._max_freq = 0.5 / self.source.sample_period
            msg = "%s: Max frequency is %s"
            LOG.debug(msg % (str(self), str(self._max_freq)))
        return self._max_freq

    @property
    def frequency(self):
        """ Frequencies represented the complex Fourier spectrum."""
        if self._frequency is None:
            self._frequency = numpy.arange(self.freq_step,
                                           self.max_freq + self.freq_step,
                                           self.freq_step)
            util.log_debug_array(LOG, self._frequency, "frequency")
        return self._frequency

    def compute_amplitude(self):
        """ Amplitude of the complex Fourier spectrum."""
        self.amplitude = numpy.abs(self.array_data)
        self.trait["amplitude"].log_debug(owner=self.__class__.__name__)

    def compute_phase(self):
        """ Phase of the Fourier spectrum."""
        self.phase = numpy.angle(self.array_data)
        self.trait["phase"].log_debug(owner=self.__class__.__name__)

    def compute_power(self):
        """ Power of the complex Fourier spectrum."""
        self.power = numpy.abs(self.array_data) ** 2
        self.trait["power"].log_debug(owner=self.__class__.__name__)

    def compute_average_power(self):
        """ Average-power of the complex Fourier spectrum."""
        self.average_power = numpy.mean(numpy.abs(self.array_data) ** 2, axis=-1)
        self.trait["average_power"].log_debug(owner=self.__class__.__name__)

    def compute_normalised_average_power(self):
        """ Normalised-average-power of the complex Fourier spectrum."""
        self.normalised_average_power = (self.average_power /
                                         numpy.sum(self.average_power, axis=0))
        self.trait["normalised_average_power"].log_debug(owner=self.__class__.__name__)

    def get_fourier_data(self, selected_state, selected_mode, normalized):
        shape = list(self.read_data_shape())

        slices = (slice(shape[0]),
                  slice(int(selected_state), min(int(selected_state) + 1, shape[1]), None),
                  slice(shape[2]),
                  slice(int(selected_mode), min(int(selected_mode) + 1, shape[3]), None))

        if normalized == "yes":
            data_matrix = self.get_data('normalised_average_power', slices)
        else:
            data_matrix = self.get_data('average_power', slices)

        data_matrix = data_matrix.reshape((shape[0], shape[2]))
        ymin = numpy.amin(data_matrix)
        ymax = numpy.amax(data_matrix)
        data_matrix = data_matrix.transpose()
        return dict(data_matrix=json.dumps(data_matrix.tolist()),
                    ymin=ymin,
                    ymax=ymax)


class WaveletCoefficients(arrays.MappedArray):
    """
    This class bundles all the elements of a Wavelet Analysis into a single
    object, including the input TimeSeries datatype and the output results as
    arrays (FloatArray)
    """
    # Overwrite attribute from superclass
    array_data = arrays.ComplexArray()

    source = time_series.TimeSeries(label="Source time-series")

    mother = basic.String(
        label="Mother wavelet",
        default="morlet",
        doc="""A string specifying the type of mother wavelet to use,
            default is 'morlet'.""")  # default to 'morlet'

    sample_period = basic.Float(label="Sample period")
    # sample_rate = basic.Integer(label = "")  inversely related

    frequencies = arrays.FloatArray(
        label="Frequencies",
        doc="A vector that maps scales to frequencies.")

    normalisation = basic.String(label="Normalisation type")
    # 'unit energy' | 'gabor'

    q_ratio = basic.Float(label="Q-ratio", default=5.0)

    amplitude = arrays.FloatArray(
        label="Amplitude",
        file_storage=core.FILE_STORAGE_EXPAND)

    phase = arrays.FloatArray(
        label="Phase",
        file_storage=core.FILE_STORAGE_EXPAND)

    power = arrays.FloatArray(
        label="Power",
        file_storage=core.FILE_STORAGE_EXPAND)

    _frequency = None
    _time = None

    __generate_table__ = True

    def configure(self):
        """After populating few fields, compute the rest of the fields"""
        # Do not call super, because that accesses data not-chunked
        self.nr_dimensions = len(self.read_data_shape())
        for i in range(self.nr_dimensions):
            setattr(self, 'length_%dd' % (i + 1), int(self.read_data_shape()[i]))

        if self.trait.use_storage is False and sum(self.get_data_shape('array_data')) != 0:
            if self.amplitude.size == 0:
                self.compute_amplitude()
            if self.phase.size == 0:
                self.compute_phase()
            if self.power.size == 0:
                self.compute_power()

    def _find_summary_info(self):
        """
        Gather scientifically interesting summary information from an instance of this datatype.
        """
        summary = {"Spectral type": self.__class__.__name__,
                   "Source": self.source.title,
                   "Wavelet type": self.mother,
                   "Normalisation": self.normalisation,
                   "Q-ratio": self.q_ratio,
                   "Sample period": self.sample_period,
                   "Number of scales": self.frequencies.shape[0],
                   "Minimum frequency": self.frequencies[0],
                   "Maximum frequency": self.frequencies[-1]}
        return summary

    @property
    def frequency(self):
        """ Frequencies represented by the wavelet spectrogram."""
        if self._frequency is None:
            self._frequency = numpy.arange(self.frequencies.lo,
                                           self.frequencies.hi,
                                           self.frequencies.step)
            util.log_debug_array(LOG, self._frequency, "frequency")
        return self._frequency

    def compute_amplitude(self):
        """ Amplitude of the complex Wavelet coefficients."""
        self.amplitude = numpy.abs(self.array_data)

    def compute_phase(self):
        """ Phase of the Wavelet coefficients."""
        self.phase = numpy.angle(self.array_data)

    def compute_power(self):
        """ Power of the complex Wavelet coefficients."""
        self.power = numpy.abs(self.array_data) ** 2

    def write_data_slice(self, partial_result):
        """
        Append chunk.
        """
        self.store_data_chunk('array_data', partial_result.array_data, grow_dimension=2, close_file=False)

        partial_result.compute_amplitude()
        self.store_data_chunk('amplitude', partial_result.amplitude, grow_dimension=2, close_file=False)

        partial_result.compute_phase()
        self.store_data_chunk('phase', partial_result.phase, grow_dimension=2, close_file=False)

        partial_result.compute_power()
        self.store_data_chunk('power', partial_result.power, grow_dimension=2, close_file=False)


class CoherenceSpectrum(arrays.MappedArray):
    """
    Result of a NodeCoherence Analysis.
    """
    # Overwrite attribute from superclass
    array_data = arrays.FloatArray(file_storage=core.FILE_STORAGE_EXPAND)

    source = time_series.TimeSeries(
        label="Source time-series",
        doc="""Links to the time-series on which the node_coherence is
            applied.""")

    nfft = basic.Integer(
        label="Data-points per block",
        default=256,
        doc="""NOTE: must be a power of 2""")

    frequency = arrays.FloatArray(label="Frequency")

    __generate_table__ = True

    def configure(self):
        """After populating few fields, compute the rest of the fields"""
        # Do not call super, because that accesses data not-chunked
        self.configure_chunk_safe()

    def _find_summary_info(self):
        """
        Gather scientifically interesting summary information from an instance of this datatype.
        """
        summary = {"Spectral type": self.__class__.__name__,
                   "Source": self.source.title,
                   "Number of frequencies": self.frequency.shape[0],
                   "Minimum frequency": self.frequency[0],
                   "Maximum frequency": self.frequency[-1],
                   "FFT length (time-points)": self.nfft}
        return summary

    def write_data_slice(self, partial_result):
        """
        Append chunk.
        """
        self.store_data_chunk('array_data', partial_result.array_data, grow_dimension=3, close_file=False)


class ComplexCoherenceSpectrum(arrays.MappedArray):
    """
    Result of a NodeComplexCoherence Analysis.
    """

    cross_spectrum = arrays.ComplexArray(
        label="The cross spectrum",
        file_storage=core.FILE_STORAGE_EXPAND,
        doc=""" A complex ndarray that contains the nodes x nodes cross
                spectrum for every frequency frequency and for every segment.""")

    array_data = arrays.ComplexArray(
        label="Complex Coherence",
        file_storage=core.FILE_STORAGE_EXPAND,
        doc="""The complex coherence coefficients calculated from the cross
                spectrum. The imaginary values of this complex ndarray represent the
                imaginary coherence.""")

    source = time_series.TimeSeries(
        label="Source time-series",
        doc="""Links to the time-series on which the node_coherence is
                applied.""")

    epoch_length = basic.Float(
        label="Epoch length",
        doc="""The timeseries was segmented into equally sized blocks
                (overlapping if necessary), prior to the application of the FFT.
                The segement length determines the frequency resolution of the
                resulting spectra.""")

    segment_length = basic.Float(
        label="Segment length",
        doc="""The timeseries was segmented into equally sized blocks
                (overlapping if necessary), prior to the application of the FFT.
                The segement length determines the frequency resolution of the
                resulting spectra.""")

    windowing_function = basic.String(
        label="Windowing function",
        doc="""The windowing function applied to each time segment prior to
                application of the FFT.""")

    __generate_table__ = True

    _frequency = None
    _freq_step = None
    _max_freq = None
    spectrum_types = ["Imaginary", "Real", "Absolute"]

    def configure(self):
        """After populating few fields, compute the rest of the fields"""
        # Do not call super, because that accesses data not-chunked
        self.configure_chunk_safe()

    def write_data_slice(self, partial_result):
        """
        Append chunk.
        """
        self.store_data_chunk('cross_spectrum', partial_result.cross_spectrum, grow_dimension=2, close_file=False)

        self.store_data_chunk('array_data', partial_result.array_data, grow_dimension=2, close_file=False)

    def _find_summary_info(self):
        """
        Gather scientifically interesting summary information from an instance of this datatype.
        """
        summary = {"Spectral type": self.__class__.__name__,
                   "Source": self.source.title,
                   "Frequency step": self.freq_step,
                   "Maximum frequency": self.max_freq,
                   "Epoch length": self.epoch_length,
                   "Segment length": self.segment_length,
                   "Windowing function": self.windowing_function
                   }
        return summary

    @property
    def freq_step(self):
        """ Frequency step size of the Complex Coherence Spectrum."""
        if self._freq_step is None:
            self._freq_step = 1.0 / self.segment_length
            msg = "%s: Frequency step size is %s"
            LOG.debug(msg % (str(self), str(self._freq_step)))
        return self._freq_step

    @property
    def max_freq(self):
        """ Maximum frequency represented in the Complex Coherence Spectrum."""
        if self._max_freq is None:
            self._max_freq = 0.5 / self.source.sample_period
            msg = "%s: Max frequency is %s"
            LOG.debug(msg % (str(self), str(self._max_freq)))
        return self._max_freq

    @property
    def frequency(self):
        """ Frequencies represented in the Complex Coherence Spectrum."""
        if self._frequency is None:
            self._frequency = numpy.arange(self.freq_step,
                                           self.max_freq + self.freq_step,
                                           self.freq_step)
        util.log_debug_array(LOG, self._frequency, "frequency")
        return self._frequency

    def get_spectrum_data(self, selected_spectrum):
        shape = list(self.read_data_shape())
        slices = (slice(shape[0]), slice(shape[1]), slice(shape[2]),)

        if selected_spectrum == self.spectrum_types[0]:
            data_matrix = self.get_data('array_data', slices).imag
            indices = numpy.triu_indices(shape[0], 1)
            data_matrix = data_matrix[indices]

        elif selected_spectrum == self.spectrum_types[1]:
            data_matrix = self.get_data('array_data', slices).real
            data_matrix = data_matrix.reshape(shape[0] * shape[0], shape[2])

        else:
            data_matrix = self.get_data('array_data', slices)
            data_matrix = numpy.absolute(data_matrix)
            data_matrix = data_matrix.reshape(shape[0] * shape[0], shape[2])

        coh_spec_sd = numpy.std(data_matrix, axis=0)
        coh_spec_av = numpy.mean(data_matrix, axis=0)

        ymin = numpy.amin(coh_spec_av - coh_spec_sd)
        ymax = numpy.amax(coh_spec_av + coh_spec_sd)

        coh_spec_sd = json.dumps(coh_spec_sd.tolist())
        coh_spec_av = json.dumps(coh_spec_av.tolist())

        return dict(coh_spec_sd=coh_spec_sd,
                    coh_spec_av=coh_spec_av,
                    ymin=ymin,
                    ymax=ymax)



# -*- coding: utf-8 -*-
#
#
# TheVirtualBrain-Framework Package. This package holds all Data Management, and
# Web-UI helpful to run brain-simulations. To use it, you also need do download
# TheVirtualBrain-Scientific Package (for simulators). See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
Adapter that uses the traits module to generate interfaces for ... Analyzer.

.. moduleauthor:: Francesca Melozzi <france.melozzi@gmail.com>
.. moduleauthor:: Marmaduke Woodman <mmwoodman@gmail.com>

"""
import numpy as np
import tvb.datatypes.time_series as time_series
from tvb.basic.traits import core, types_basic, util
from tvb.basic.logger.builder import get_logger
from scipy.spatial.distance import pdist
from sklearn.manifold import SpectralEmbedding
from sklearn.cluster import DBSCAN
from numpy import linalg


LOG = get_logger(__name__)


class FcdCalculator(core.Type):
    """
    The present class will do the following actions:

    - Compute the the fcd of the timeseries; the fcd is calculated in the following way:
        the time series is divided in time window of fixed length and with an overlapping of fixed length.
        The data-points within each window, centered at time ti, are used to calculate FC(ti) as Pearson correlation.
        The ij element of the FCD matrix is calculated as the Pearson correlation between FC(ti) and FC(tj) -in a vector
    - Apply to the fcd the spectral embedding algorithm in order to calculate epochs of stability of the fcd
        (length of time during which FC matrix are high correlated).

    The algorithm can produce 2 kind of results:

    - case 1: the algorithm is able to identify the epochs of stability
        -- fcs calculated over the epochs of stability (excluded the first one = artifact, due to initial conditions)
        -- 3 eigenvectors, associated to the 3 largest eigenvalues, of the fcs are extracted
    - case 2: the algorithm is not able to identify the epochs of stability
        -- fc over the all time series is calculated
        -- 3 first eigenvectors, associated to the 3 largest eigenvalues, of the fcs are extracted

    :return
        - fcd matrix whose values are between -1 and 1, inclusive.
        - in case 1: fcd matrix segmented i.e. fcd whose values are between -1 and 1.1, inclusive.
            (Value=1.1 for time not belonging to epochs of stability identified with spectral embedding algorithm)
            in case 2: fcd matrix segmented identical to the fcd matrix not segmented
        - dictionary containing the eigenvectors.
        - dictionary containing the eigenvalues
        - connectivity associated to the TimeSeriesRegions

    """

    time_series = time_series.TimeSeriesRegion(
        label="Time Series",
        required=True,
        doc="""The time-series for which the fcd matrices are calculated.""")

    sw = types_basic.Float(
        label="Sliding window length (ms)",
        default=120000,
        doc="""Length of the time window used to divided the time series.
        FCD matrix is calculated in the following way: the time series is divided in time window of fixed length and
        with an overlapping of fixed length. The datapoints within each window, centered at time ti, are used to
        calculate FC(ti) as Pearson correlation. The ij element of the FCD matrix is calculated as the Pearson
        Correlation between FC(ti) and FC(tj) arranged in a vector.""")

    sp = types_basic.Float(
        label="Spanning between two consecutive sliding window (ms)",
        default=2000,
        doc="""Spanning= (time windows length)-(overlapping between two consecutive time window). FCD matrix is
        calculated in the following way: the time series is divided in time window of fixed length and with an
        overlapping of fixed length. The datapoints within each window, centered at time ti, are used to calculate
        FC(ti) as Pearson Correlation. The ij element of the FCD matrix is calculated as the Pearson correlation
        between FC(ti) and FC(tj) arranged in a vector""")

    def evaluate(self):
        cls_attr_name = self.__class__.__name__ + ".time_series"
        self.time_series.trait["data"].log_debug(owner=cls_attr_name)

        # Pass sp and sw in the right time reference (means considering the sample period)
        sp = float(self.sp) / self.time_series.sample_period
        sw = float(self.sw) / self.time_series.sample_period

        input_shape = self.time_series.read_data_shape()
        result_shape = self.result_shape(input_shape)

        fcd = np.zeros(result_shape)
        fc_stream = {}  # dict where the fc calculated over the sliding window will be stored
        start = -sp  # in order to well initialize the first starting point of the FC stream
        for mode in range(result_shape[3]):
            for var in range(result_shape[2]):
                for nfcd in range(result_shape[0]):
                    start += sp
                    current_slice = tuple([slice(int(start), int(start+sw) + 1), slice(var, var + 1),
                                           slice(input_shape[2]), slice(mode, mode + 1)])
                    data = self.time_series.read_data_slice(current_slice).squeeze()
                    fc = np.corrcoef(data.T)
                    # the triangular part of the fc is organized as a vector, excluding the diagonal (always ones)
                    triangular = np.triu_indices(len(fc), 1)
                    fc_stream[nfcd] = fc[triangular]
                for i in range(result_shape[0]):
                    j = i
                    while j < result_shape[0]:
                        fci = fc_stream[i]
                        fcj = fc_stream[j]
                        fcd[i, j, var, mode] = np.corrcoef(fci, fcj)[0, 1]
                        fcd[j, i, var, mode] = fcd[i, j, var, mode]
                        j += 1

        util.log_debug_array(LOG, fcd, "FCD")

        num_eig = 3  # number of the eigenvector that will be extracted

        eigvect_dict = {}  # holds eigenvectors of the fcs calculated over the epochs, key1=mode, key2=var, key3=numb ep
        eigval_dict = {}  # holds eigenvalues of the fcs calculated over the epochs, key1=mode, key2=var, key3=numb ep
        for mode in range(result_shape[3]):
            eigvect_dict[mode] = {}
            eigval_dict[mode] = {}
            for var in range(result_shape[2]):
                eigvect_dict[mode][var] = {}
                eigval_dict[mode][var] = {}
                fcd_matrix = fcd[:, :, var, mode]
                [xir, xir_cutoff] = spectral_embedding(fcd_matrix)
                epochs_extremes = epochs_interval(xir, xir_cutoff, sp, sw)
                fcd_segmented = fcd.copy()
                if epochs_extremes.shape[0] <= 1:
                    # means that there are no more than 1 epochs of stability, thus the eigenvectors of
                    # the FC calculated over the entire TimeSeries will be calculated
                    epochs_extremes = np.zeros((2, 2), dtype=float)
                    epochs_extremes[1, 1] = input_shape[0]  # [0,0] set in order to skip the first epoch
                else:
                    # means that more than 1 epochs of stability is identified thus fcd_segmented is calculated
                    fcd_segmented[xir > xir_cutoff, :, var, mode] = 1.1
                    fcd_segmented[:, xir > xir_cutoff, var, mode] = 1.1

                for ep in range(1, epochs_extremes.shape[0]):
                    eigvect_dict[mode][var][ep] = []
                    eigval_dict[mode][var][ep] = []
                    current_slice = tuple([slice(int(epochs_extremes[ep][0]), int(epochs_extremes[ep][1]) + 1),
                                           slice(var, var + 1), slice(input_shape[2]), slice(mode, mode + 1)])
                    data = self.time_series.read_data_slice(current_slice).squeeze()
                    fc = np.corrcoef(data.T)  # calculate fc over the epoch of stability
                    eigval_matrix, eigvect_matrix = linalg.eig(fc)
                    eigval_matrix = np.real(eigval_matrix)
                    eigvect_matrix = np.real(eigvect_matrix)
                    eigval_matrix = eigval_matrix / np.sum(np.abs(eigval_matrix))  # normalize eigenvalues to [0 and 1)
                    for en in range(num_eig):
                        index = np.argmax(eigval_matrix)
                        eigvect_dict[mode][var][ep].append(abs(eigvect_matrix[:, index]))
                        eigval_dict[mode][var][ep].append(eigval_matrix[index])
                        eigval_matrix[index] = 0

        return [fcd, fcd_segmented, eigvect_dict, eigval_dict, self.time_series.connectivity]


    def result_shape(self, input_shape):
        """Returns the shape of the fcd"""
        sp = float(self.sp) / self.time_series.sample_period
        sw = float(self.sw) / self.time_series.sample_period
        fcd_points = int((input_shape[0] - sw) / sp)
        result_shape = (fcd_points, fcd_points, input_shape[1], input_shape[3])
        return result_shape


    def result_size(self, input_shape):
        """
        Returns the storage size in Bytes of the main result of .
        """
        result_size = np.sum(map(np.prod, self.result_shape(input_shape))) * 8.0  # Bytes
        return result_size


    def extended_result_size(self, input_shape):
        """
        Returns the storage size in Bytes of the extended result of the ....
        That is, it includes storage of the evaluated ... attributes
        such as ..., etc.
        """
        extend_size = self.result_size(input_shape)  # Currently no derived attributes.
        return extend_size


# Methods:
def spectral_dbscan(fcd, n_dim=2, eps=0.3, min_samples=50):
    fcd = fcd - fcd.min()
    se = SpectralEmbedding(n_dim, affinity="precomputed")
    xi = se.fit_transform(fcd)
    pd = pdist(xi)
    eps = np.percentile(pd, 100 * eps)
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(xi)
    return xi.T, db.labels_


def compute_radii(xi, centered=False):
    if centered:
        xi = xi.copy() - xi.mean(axis=1).reshape((len(xi), 1))
    radii = np.sqrt(np.sum(xi ** 2, axis=0))
    return radii


def spectral_embedding(fcd):
    xi, _ = spectral_dbscan(fcd, 2)
    xir = compute_radii(xi, True)
    xir_sorted = np.sort(xir)
    xir_cutoff = 0.5 * xir_sorted[-1]
    return xir, xir_cutoff


def epochs_interval(xir, xir_cutoff, sp, sw):
    # Calculate the starting point and the ending point of each epoch of stability
    # sp=spanning, sw=sliding window
    epochs_dict = {}  # here the starting and the ending point will be stored
    thresholds = np.where(xir < xir_cutoff)
    tt = 0
    ep = 0
    while (tt + 2) < len(thresholds[0]):
        epochs_dict[ep] = [thresholds[0][tt]]  # starting point of epoch ep
        while ((tt + 2) != len(thresholds[0])) & (thresholds[0][tt + 1] == thresholds[0][tt] + 1):
            # until the vector is not finish and until each entries +1 is equal to the next one
            tt += 1
        epochs_dict[ep].append(thresholds[0][tt])
        tt += 1
        ep += 1
    # The relation between the index of the fcd[T] and the time point [t(i)] of the BOLD is the following:
    # T=0 indicates the FC calculate over the length of time that starts at (t=0) and that
    #     ends at (t=0)+sw (sw=length of the sliding window, sp=spanning between sliding windows)
    # T=1 indicates the FC calculate over the length of time that starts at (t=0)+sp and that ends at (t=0)+sp+sw
    # T=2 indicates the FC calculate over the length of time that starts at (t=0)+2*sp and that ends at (t=0)+s*sp+sw
    # Thus we can write:
    # [interval of T=0]=[t(0)] U [t(0)+sw]
    # [interval of T=1]=[t(0)+sp] U [t(0)+sp+sw]
    # [interval of T=2]=[t(0)+2*sp] U [t(0)+2*sp+sw]
    # ...
    # [interval of T=i]=[t(0)+i*sp] U [t(0)+i*sp+sw]
    # Once we have the interval of the Epoch of stability that starts at T=s and ends at T=f
    # we want to calculate the FC taking the BOLD that starts and ends respectively at:
    # t(0)+s*sp; t(0)+f*sp+sw
    # Thus (we save the BOLD time in the epochs_extremes matrix)
    epochs_extremes = np.zeros((len(epochs_dict), 2), dtype=float)
    for ep in range(len(epochs_dict)):
        epochs_extremes[ep, 0] = epochs_dict[ep][0] * sp
        epochs_extremes[ep, 1] = epochs_dict[ep][1] * sp + sw
    return epochs_extremes



"""
XML Test Runner for PyUnit
"""

# Written by Sebastian Rittau <srittau@jroger.in-berlin.de> and placed in
# the Public Domain. With contributions by Paolo Borelli and others.
# Updated by Lia Domide <lia.domide@codemart.ro> by adding report of skipped tests.

from __future__ import with_statement

__version__ = "0.1"

import sys
import time
import traceback
import unittest
from xml.sax.saxutils import escape
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO



class _TestInfo(object):
    """
    Information about a particular test.
    Used by _XMLTestResult.
    """

    def __init__(self, test, time):
        (self._class, self._method) = test.id().rsplit(".", 1)
        self._time = time
        self._error = None
        self._failure = None
        self._skipped = None

    @staticmethod
    def create_success(test, time):
        """Create a _TestInfo instance for a successful test."""
        return _TestInfo(test, time)

    @staticmethod
    def create_failure(test, time, failure):
        """Create a _TestInfo instance for a failed test."""
        info = _TestInfo(test, time)
        info._failure = failure
        return info

    @staticmethod
    def create_error(test, time, error):
        """Create a _TestInfo instance for an erroneous test."""
        info = _TestInfo(test, time)
        info._error = error
        return info

    @staticmethod
    def create_skipped(test, time, skipped):
        """Create a _TestInfo instance for a skipped test."""
        info = _TestInfo(test, time)
        info._skipped = skipped
        return info

    def print_report(self, stream):
        """ Print information about this test case in XML format to the supplied stream. """
        stream.write('  <testcase classname="%(class)s" name="%(method)s" time="%(time).4f">' %
                     {"class": self._class,
                      "method": self._method,
                      "time": self._time
                      })
        if self._failure is not None:
            self._print_error(stream, 'failure', self._failure)
        if self._error is not None:
            self._print_error(stream, 'error', self._error)
        if self._skipped is not None:
            self._print_skipped(stream, self._skipped)
        stream.write('</testcase>\n')

    def _print_error(self, stream, tagname, error):
        """Print information from a failure or error to the supplied stream."""
        text = escape(str(error[1]))
        stream.write('\n')
        stream.write('    <%s type="%s">%s\n' % (tagname, _clsname(error[0]), text))
        tb_stream = StringIO()
        traceback.print_tb(error[2], None, tb_stream)
        stream.write(escape(tb_stream.getvalue()))
        stream.write('    </%s>\n' % tagname)
        stream.write('  ')
        
    def _print_skipped(self, stream, skip_reason):
        """ Print the skip reason"""
        stream.write('\n')
        stream.write('    <skipped reason="%s"/>\n' % str(skip_reason))
        stream.write('  ')



def _clsname(cls):
    return cls.__module__ + "." + cls.__name__



class _XMLTestResult(unittest.TestResult):
    """
    A test result class that stores result as XML.
    Used by XMLTestRunner.
    """

    def __init__(self, classname):
        unittest.TestResult.__init__(self)
        self._test_name = classname
        self._start_time = None
        self._tests = []
        self._error = None
        self._failure = None
        self._skipped = None

    def startTest(self, test):
        unittest.TestResult.startTest(self, test)
        self._error = None
        self._failure = None
        self._skipped = None
        self._start_time = time.time()

    def stopTest(self, test):
        time_taken = time.time() - self._start_time
        unittest.TestResult.stopTest(self, test)
        if self._error:
            info = _TestInfo.create_error(test, time_taken, self._error)
        elif self._failure:
            info = _TestInfo.create_failure(test, time_taken, self._failure)
        elif self.skipped:
            info = _TestInfo.create_skipped(test, time_taken, self._skipped)
        else:
            info = _TestInfo.create_success(test, time_taken)
        self._tests.append(info)

    def addError(self, test, err):
        unittest.TestResult.addError(self, test, err)
        self._error = err

    def addFailure(self, test, err):
        unittest.TestResult.addFailure(self, test, err)
        self._failure = err

    def addSkip(self, test, reason):
        unittest.TestResult.addSkip(self, test, reason)
        self._skipped = reason

    def print_report(self, stream, logs_stream, time_taken, out, err):
        """
        Prints the XML report to the supplied stream.
        The time the tests took to perform as well as the captured standard
        output and standard error streams must be passed in.
        """
        stream.write('<testsuite errors="%(e)d" failures="%(f)d" skipped="%(s)d" ' % {"e": len(self.errors),
                                                                                      "f": len(self.failures),
                                                                                      "s": len(self.skipped)})
        stream.write('name="%(n)s" tests="%(t)d" time="%(time).3f">\n' %
                     {"n": self._test_name,
                      "t": self.testsRun,
                      "time": time_taken,
                      })
        for info in self._tests:
            info.print_report(stream)

        stream.write('</testsuite>\n')

        logs_stream.write("OUTPUT: \n \n\n")
        logs_stream.write(out)
        logs_stream.write("\n\n\n ERRORS: \n \n\n")
        logs_stream.write(err)



class XMLTestRunner(object):
    """
    A test runner that stores results in XML format compatible with JUnit.

    XMLTestRunner(stream=None) -> XML test runner

    The XML file is written to the supplied stream. If stream is None, the
    results are stored in a file called TEST-<module>.<class>.xml in the
    current working directory (if not overridden with the path property),
    where <module> and <class> are the module and class name of the test class.
    """

    def __init__(self, stream, logs_stream):
        self._stream = stream
        self._logs_stream = logs_stream
        self._path = "."


    def run(self, test):
        """Run the given test case or test suite."""
        class_ = test.__class__
        classname = class_.__module__ + "." + class_.__name__

        result = _XMLTestResult(classname)
        start_time = time.time()

        with _fake_std_streams():
            test(result)
            try:
                out_s = sys.stdout.getvalue()
            except AttributeError:
                out_s = ""
            try:
                err_s = sys.stderr.getvalue()
            except AttributeError:
                err_s = ""

        time_taken = time.time() - start_time
        result.print_report(self._stream, self._logs_stream, time_taken, out_s, err_s)

        return result

    def _set_path(self, path):
        self._path = path

    path = property(lambda self: self._path, _set_path, None, """The path where the XML files are stored.
    This property is ignored when the XML file is written to a file stream.""")



class _fake_std_streams(object):

    def __enter__(self):
        self._orig_stdout = sys.stdout
        self._orig_stderr = sys.stderr
        sys.stdout = StringIO()
        sys.stderr = StringIO()

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.stdout = self._orig_stdout
        sys.stderr = self._orig_stderr



if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#
"""
Created on Mar 20, 2013

.. moduleauthor:: Bogdan Neacsa <bogdan.neacsa@codemart.ro>
"""
if __name__ == "__main__":
    from tvb.tests.library import setup_test_console_env
    setup_test_console_env()
    
    
import numpy
import unittest

from tvb.datatypes import arrays
from tvb.tests.library.base_testcase import BaseTestCase
        
class ArraysTest(BaseTestCase):
    
    
    def test_float_array(self):
        """
        Create a float array, check that shape is correct.
        """
        data = numpy.random.random((10, 10))
        array_dt = arrays.FloatArray()
        array_dt.data = data
        self.assertEqual(array_dt.shape, (10, 10))
        
        
    def test_integer_array(self):
        """
        Create an integer array, check that shape is correct.
        """
        data = numpy.arange(100, dtype=int)
        array_dt = arrays.IntegerArray()
        array_dt.data = data
        self.assertEqual(array_dt.shape, (100,))
        
        
    def test_complex_array(self):
        """
        Create a complex array, check that shape is correct.
        """
        data = numpy.array([numpy.complex(100, 2) for _ in range(100)])
        array_dt = arrays.ComplexArray()
        array_dt.data = data
        self.assertEqual(array_dt.shape, (100,))
        
        
    def test_bool_array(self):
        """
        Create a boolean array, check that shape is correct.
        """
        data = numpy.array([[False for _ in range(12)] for _ in range(10)])
        array_dt = arrays.ComplexArray()
        array_dt.data = data
        self.assertEqual(array_dt.shape, (10, 12))
        
    
    def test_string_array(self):
        """
        Create a string array, check that shape is correct.
        """
        data = numpy.array([['test' for _ in range(12)] for _ in range(10)])
        array_dt = arrays.StringArray()
        array_dt.data = data
        self.assertEqual(array_dt.shape, (10, 12))
        
        
    def test_position_array(self):
        """
        Create a position array, check that shape is correct.
        """
        data = numpy.random.random((10, 10))
        array_dt = arrays.PositionArray(coordinate_system="test_system",
                                        coordinate_space="test_space")
        array_dt.data = data
        self.assertEqual(array_dt.shape, (10, 10))
        self.assertEqual(array_dt.coordinate_space, "test_space")
        self.assertEqual(array_dt.coordinate_system, "test_system")
        
        
    def test_orientation_array(self):
        """
        Create an orientation array, check that shape is correct.
        """
        data = numpy.random.random((10, 10))
        array_dt = arrays.OrientationArray(coordinate_system_or="test_system")
        array_dt.data = data
        self.assertEqual(array_dt.shape, (10, 10))
        self.assertEqual(array_dt.coordinate_system_or, "test_system")
        
        
    def test_index_array(self):
        """
        Create an index array, check that shape is correct.
        """
        target_data = numpy.random.random((10, 3))
        target_array = arrays.FloatArray()
        target_array.data = target_data
        array_dt = arrays.IndexArray(target=target_array)
        array_dt.data = numpy.arange(30).reshape((10, 3))
        self.assertEqual(array_dt.shape, (10, 3))
        self.assertEqual(array_dt.target.shape, (10, 3))
        
        
def suite():
    """
    Gather all the tests in a test suite.
    """
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(ArraysTest))
    return test_suite


if __name__ == "__main__":
    #So you can run tests from this package individually.
    TEST_RUNNER = unittest.TextTestRunner()
    TEST_SUITE = suite()
    TEST_RUNNER.run(TEST_SUITE) 


# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
A collection of plotting functions used by simulator/demos

import an usage for MAYAVI based plots should look like::
    
    from tvb.simulator.plot.mayavi_tools import *
    
    if IMPORTED_MAYAVI:
        plt = plot_function(...)

.. moduleauthor:: Stuart A. Knock <Stuart@tvb.invalid>
.. moduleauthor:: Paula Sanz Leon <paula.sanz-leon@univ-amu.fr>
"""

import numpy
import scipy as sp
import networkx as nx
from tvb.basic.logger.builder import get_logger

LOG = get_logger(__name__)

#import pdb; pdb.set_trace()
##----------------------------------------------------------------------------##
##-                   mayavi based plotting functions                        -##
##----------------------------------------------------------------------------##
try:
    from mayavi import mlab

    IMPORTED_MAYAVI = True
except ImportError:
    LOG.warning("Mayavi is needed for this demo but due to sizing and packaging constraints we are not distributing it. "
              "If you want to see the actual plot you should use the github version and install all the required "
              "dependencies as described here: (advanced users only)"
              "http://docs.thevirtualbrain.com/manuals/ContributorsManual/ContributorsManual.html#the-unaided-setup")
    IMPORTED_MAYAVI = False
    #raise

if IMPORTED_MAYAVI:
    @mlab.animate(delay=41, ui=True)
    def surface_timeseries(surface, data, step=1):
        """
        
        """
        fig = mlab.figure(figure="surface_timeseries", fgcolor=(0.5, 0.5, 0.5))
        #Plot an initial surface and colourbar #TODO: Change to use plot_surface function, see below.
        surf_mesh = mlab.triangular_mesh(surface.vertices[:, 0],
                                         surface.vertices[:, 1],
                                         surface.vertices[:, 2],
                                         surface.triangles,
                                         scalars=data[0, :],
                                         vmin=data.min(), vmax=data.max(),
                                         figure=fig)
        mlab.colorbar(object=surf_mesh, orientation="vertical")

        #Handle for the surface object and figure
        surf = surf_mesh.mlab_source

        #Time #TODO: Make actual time rather than points, where/if possible.
        tpts = data.shape[0]
        time_step = mlab.text(0.85, 0.125, ("0 of %s" % str(tpts)),
                              width=0.0625, color=(1, 1, 1), figure=fig,
                              name="counter")

        #Movie
        k = 0
        while 1:
            if abs(k) >= tpts:
                k = 0
            surf.set(scalars=data[k, :])
            time_step.set(text=("%s of %s" % (str(k), str(tpts))))
            k += step
            yield
        mlab.show()
        #--------------------------------------------------------------------------#


    #TODO: Make, posssibly with a wrapper function, to work directly with 
    #      SurfacePattern object... Inner function name plot_surface

    def plot_surface(surface, fig=None, name=None, op=1.0, rep='surface'):
        """
        """
        if fig is None:
            fig = mlab.figure(figure=name, fgcolor=(0.5, 0.5, 0.5))

        surf_mesh = mlab.triangular_mesh(surface.vertices[:, 0],
                                         surface.vertices[:, 1],
                                         surface.vertices[:, 2],
                                         surface.triangles,
                                         color=(0.7, 0.67, 0.67),
                                         opacity=op,
                                         representation=rep,
                                         figure=fig)

        return surf_mesh


    def surface_orientation(surface, normals="triangles", name=None):
        """
        """
        fig = mlab.figure(figure=name, fgcolor=(0.5, 0.5, 0.5))
        surf_mesh = mlab.triangular_mesh(surface.vertices[:, 0],
                                         surface.vertices[:, 1],
                                         surface.vertices[:, 2],
                                         surface.triangles,
                                         color=(0.7, 0.67, 0.67),
                                         figure=fig)
        if normals == "triangles":
            surf_orient = mlab.quiver3d(surface.triangle_centres[:, 0],
                                        surface.triangle_centres[:, 1],
                                        surface.triangle_centres[:, 2],
                                        surface.triangle_normals[:, 0],
                                        surface.triangle_normals[:, 1],
                                        surface.triangle_normals[:, 2])
        elif normals == "vertices":
            surf_orient = mlab.quiver3d(surface.vertices[:, 0],
                                        surface.vertices[:, 1],
                                        surface.vertices[:, 2],
                                        surface.vertex_normals[:, 0],
                                        surface.vertex_normals[:, 1],
                                        surface.vertex_normals[:, 2])
        else:
            LOG.error("normals must be either 'triangles' or 'vertices'")

        return (surf_mesh, surf_orient)


    def surface_parcellation(cortex_boundaries, colouring, mapping_colours, colour_rgb, interaction=False):
        """
        """
        number_of_vertices = cortex_boundaries.cortex.vertices.shape[0]
        number_of_triangles = cortex_boundaries.cortex.triangles.shape[0]

        number_of_regions = len(cortex_boundaries.region_neighbours)
        alpha = 255
        lut = numpy.zeros((number_of_regions, 4), dtype=numpy.uint8)
        for k in range(number_of_regions):
            lut[k] = numpy.hstack((colour_rgb[mapping_colours[colouring[k]]], alpha))

        fig = mlab.figure(figure="surface parcellation", bgcolor=(0.0, 0.0, 0.0), fgcolor=(0.5, 0.5, 0.5))
        surf_mesh = mlab.triangular_mesh(cortex_boundaries.cortex.vertices[:number_of_vertices//2, 0],
                                         cortex_boundaries.cortex.vertices[:number_of_vertices//2, 1],
                                         cortex_boundaries.cortex.vertices[:number_of_vertices//2, 2],
                                         cortex_boundaries.cortex.triangles[:number_of_triangles//2, :],
                                         scalars=cortex_boundaries.cortex.region_mapping[:number_of_vertices//2],
                                         figure=fig)
        surf_mesh.module_manager.scalar_lut_manager.lut.number_of_colors = number_of_regions
        #surf_mesh.module_manager.scalar_lut_manager.lut.table = lut

        #TODO: can't get region labels to associate with colorbar...
        #mlab.colorbar(object=surf_mesh, orientation="vertical")
        x = cortex_boundaries.boundary[:, 0]
        y = cortex_boundaries.boundary[:, 1]
        z = cortex_boundaries.boundary[:, 2]
        bpts = mlab.points3d(x, y, z, color=(0.25, 0.25, 0.25), scale_factor=1)
        mlab.show(stop=interaction)
        return surf_mesh, bpts


    def surface_pattern(surface, vertex_colours, custom_lut = None, foci=None):
        """
        Plot a surface and colour it based on a vector of length number of 
        vertices (vertex_colours).

        * How to obtain a pretty picture (from Mayavi's gui): 
        
          - set surf_mesh color to rgb(237, 217, 221)
          - add a surface module derived from surf_mesh; set 'Actor' 
            representation to wireframe; colour 'gray'.
          - enable contours of scalar_surf  
        """
        #surf_mesh = plot_surface(surface, name="surface pattern")
        fig = mlab.figure(figure="surface pattern", fgcolor=(0.5, 0.5, 0.5))
        surf_mesh = mlab.triangular_mesh(surface.vertices[:, 0],
                                         surface.vertices[:, 1],
                                         surface.vertices[:, 2],
                                         surface.triangles,
                                         figure=fig)
        sm_obj = surf_mesh.mlab_source
        scalar_data = surf_mesh.mlab_source.dataset.point_data
        scalar_data.scalars = vertex_colours
        scalar_data.scalars.name = 'Scalar data'
        scalar_data.update()
        scalar_mesh = mlab.pipeline.set_active_attribute(surf_mesh, point_scalars='Scalar data')
        scalar_surf = mlab.pipeline.surface(scalar_mesh)


        if custom_lut is not None:

            # and finally we put this LUT back in the surface object. We could have
            # added any 255*4 array rather than modifying an existing LUT.
            scalar_surf.module_manager.scalar_lut_manager.lut.table = custom_lut
        
        if foci is not None:
            pts = mlab.points3d(foci[:,0], 
                                foci[:,1], 
                                foci[:,2],
                            scale_factor = 2.,
                            scale_mode = 'none',
                            resolution = 5,
                            opacity=0.01)

        
        mlab.show(stop=True)
        return sm_obj


    def xmas_balls(connectivity, 
                 labels=True, labels_indices=None, 
                 balls_colormap='Blues', 
                 bgcolor = (0.5, 0.5, 0.5),
                 node_data=None, node_size=4.,
                 edge_data=True, edge_color=(0.8, 0.8, 0.8), edge_size=0.2,
                 text_size=0.042, text_color=(0, 0, 0),
                 remove_nodes=False, nbunch=[], 
                 remove_edges=False, ebunch=[]):
        """
        Plots coloured balls at the region centres of connectivity, colour and
        size is determined by a vector of length number of regions (node_data).
        
        Optional: adds the connections between pair of nodes.
        
        """

        mlab.figure(1, bgcolor=bgcolor)

        # Get graph
        G = nx.from_numpy_matrix(numpy.matrix(connectivity.weights))

        # Get the subgraph of nodes in nbunch
        if remove_nodes:
            G.remove_nodes_from([n for n in G if n not in set(nbunch)])
            #G.remove_nodes_from([node for node,degree in G.degree().items() if degree < 2])

        if remove_edges:
            G.remove_edges_from([e for e in G.edges() if e not in ebunch])


        # scalar colors
        if node_data is not None:
            scalars = node_data
            #mlab.colorbar(orientation="vertical")
        else:
            scalars = numpy.array(G.nodes())*20

        pts = mlab.points3d(connectivity.centres[:,0], 
                            connectivity.centres[:,1], 
                            connectivity.centres[:,2],
                            scalars,
                            #mask_points=1,
                            scale_factor = node_size,
                            scale_mode = 'none',
                            colormap = balls_colormap,
                            resolution = 5,
                            opacity=0.01)

        if labels:
            if labels_indices is not None:
                for i, (idx) in enumerate(labels_indices):
                    x = connectivity.centres[idx, 0]
                    y = connectivity.centres[idx, 1]
                    z = connectivity.centres[idx, 2]
                    label = mlab.text(x, y, connectivity.region_labels[idx], 
                              z=z,
                              width=text_size, 
                              name=str(connectivity.region_labels[idx]), 
                              color=text_color)
                    label.property.shadow = False
            else:
                for i, (x, y, z) in enumerate(connectivity.centres):
                    label = mlab.text(x, y, connectivity.region_labels[i], 
                              z=z,
                              width=text_size, 
                              name=str(connectivity.region_labels[i]), 
                              color=text_color)
                    label.property.shadow = False

        if edge_data:
            pts.mlab_source.dataset.lines = numpy.array(G.edges())
            tube = mlab.pipeline.tube(pts, tube_radius = edge_size)
            mlab.pipeline.surface(tube, color=edge_color, representation='wireframe', opacity=0.3)

        #mlab.show()

            # stop the scene
            #mlab.show(stop=True)


    def connectivity_3d(connectivity, order=None, edge_cutoff=None):
        """
        Plots a 3D representation of the delayed-connectivity structure.
        See Fig. 3 in (Knock et al 2009)

        [Nodes x Nodes x Delays]
        
        Original script can be found at: 
        BrainNetworkModels_3.1/PlottingTools/PlotConnectivity3D.m
        
        """

        fig = mlab.figure(figure="Connectivity 3D", bgcolor=(0.0, 0.0, 0.0))

        N = connectivity.number_of_regions // 2
        minW = connectivity.weights.min()
        maxW = connectivity.weights.max()

        if connectivity.delays is None:
            connectivity.configure()

        minD = connectivity.delays.min()
        maxD = connectivity.delays.max()
        stepD = (maxD - minD) / 10.
        if order is None:
            order = numpy.arange(0, N)

        if edge_cutoff is None:
            edge_cutoff = minW

        # colourmap to emphasise large numbers
        #MAP = numpy.loadtxt('../plot/colourmaps/BlackToBlue')
        #mapstep = 1. / MAP.shape[0]

        # Loop over connectivity matrix, colouring and one cube per matrix element
        K = []
        D = []
        M = []
        S = []
        for k in range(N):
            for m in range(N):
                if connectivity.weights[k, m] != 0:
                    if k != m:
                        #not self connection (diagonal)
                        if connectivity.weights[k, m] > edge_cutoff:
                            K.append(k + 2.)
                            D.append(connectivity.delays[k, m] + stepD)
                            M.append(m + 2.0)
                            S.append(connectivity.weights[k, m])
        mlab.points3d(K, D, M, S, mode='cube')
        mlab.show(stop=True)


        #--------------------------------------------------------------------------#

if __name__ == '__main__':
    # Do some stuff that tests or makes use of this module... 
    pass


##- EoF -##



# -*- coding: utf-8 -*-
#
#
# TheVirtualBrain-Framework Package. This package holds all Data Management, and 
# Web-UI helpful to run brain-simulations. To use it, you also need do download
# TheVirtualBrain-Scientific Package (for simulators). See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#

"""
A data model for the input trees.
.. moduleauthor:: Mihai Andrei <mihai.andrei@codemart.ro>
"""

def _repr_help_format_list(lines):
    'Formats a list, trying to keep indentation. It makes repr behave similar to pprint.pformat'
    lines = '\n'.join(str(o) for o in lines)
    lines = lines.splitlines()
    lines = '\n    '.join(lines)
    return lines


class SelectTypeNode(object):
    '''
    This node represents a collection of non mapped Types.
    Examples Model collection, Integrator collection
    '''
    def __init__(self, name, select_multiple, options, default=None, label=None, description=None):
        self.name = name
        self.label = label or name
        self.description = description or self.label
        self.type = 'selectMultiple' if select_multiple else 'select'
        self.options = options # of type TypeNode
        if default is None and len(options):
            self.default = options[0].name
        else:
            self.default = default


    def __repr__(self):
        opts = _repr_help_format_list(self.options)
        return ('SelectTypeNode(name=%r, type=%r, default=%r, label=%r,\n'
                '  options=[\n    %s    \n])' % (self.name, self.type, self.default, self.label, opts))


class TypeNode(object):
    '''
    A non-mapped type. Has attributes
    '''
    def __init__(self, name, value, class_, description, attributes):
        self.name = name  # _ui_name
        self.value = value   # short class name
        self.type = class_ # fqn class
        self.description = description # docstring
        self.attributes = attributes

    def __repr__(self):
        attrs = _repr_help_format_list(self.attributes)
        return ('%s(name=%r, class=%r, \n'
                '  attributes=[\n    %s    \n])' %(type(self).__name__, self.name, self.type, attrs))


class LeafNode(object):
    '''
    Node has no descendants
    '''
    def __init__(self, name, type_, default=None, label=None, description=None, required=True):
        self.name = name
        self.type = type_
        self.label = label or name
        self.description = description or self.label
        self.required = required
        self.default = default

    def __repr__(self):
        return '%s(name=%r, type=%r, label=%r, default=%r)' % (type(self).__name__, self.name, self.type, self.label, self.default)

#class NodeWithInstancesFromDb
class DatatypeNode(object):
    '''
    A dataype node. Has filters.
    '''
    def __init__(self, name, type_, label=None, description=None, required=True, conditions=None, filters_ui=None):
        '''
        type_ may be a FQN or a type
        '''
        if not isinstance(type_, basestring):
            type_ = type_.__module__ + '.' + type_.__name__
        self.name = name
        self.type = type_
        self.label = label or name
        self.description = description or self.label
        self.required = required
        self.conditions = conditions
        self.filters_ui = filters_ui
        self.default = None
        # List of concrete instance gid's. These Options are runtime values, filled from the db after applying filters.
        # This is not a Type level concept, so setting them in a get_input_tree makes no sense.
        self.options = []

    def __repr__(self):
        return '%s(name=%r, type=%r, label=%r, default=%r)' % (
        type(self).__name__, self.name, self.type, self.label, self.default)


class ComplexDtypeNode(object):
    '''
    Usually a cortex. It is not a LeafNode even though similar to a datatypenode
    '''
    def __init__(self, name, type_, attributes, label=None, description=None, required=True, conditions=None, filters_ui=None):
        self.name = name
        self.type = type_
        self.description = description
        self.attributes = attributes
        self.label = label
        self.required = required
        self.conditions = conditions
        self.filters_ui = filters_ui

    def __repr__(self):
        attrs = _repr_help_format_list(self.attributes)
        return ('%s(name=%r, class=%r, \n'
                '  attributes=[\n    %s    \n]\n)' %(type(self).__name__, self.name, self.type, attrs))


class Range(object):
    def __init__(self, min_, max_, step):
        self.min = min_
        self.max = max_
        self.step = step

    def __repr__(self):
        return 'Range(%d, %d, %d)'% (self.min, self.max, self.step)


class NumericNode(LeafNode):
    def __init__(self, name, type_, default=0, range_=None,
                 label=None, description=None, required=True):
        LeafNode.__init__(self, name, type_, default, label, description, required)
        self.range = range_

    def __repr__(self):
        return '%s(name=%r, type=%r, label=%r, default=%r, range=%r)' % (
            type(self).__name__, self.name, self.type, self.label, self.default, self.range)


class ArrayNode(NumericNode):
    def __init__(self, name, type_, default='[]', range_=None,
                 label=None, description=None, required=True):
        NumericNode.__init__(self, name, type_, default, range_, label, description, required)


class DictNode(object):
    def __init__(self, name, type_, attributes, default):
        self.name = name
        self.label = name
        self.type = type_  # todo: is it useful to keep the concrete leaf node type?
        self.default = default
        self.attributes = attributes # type Leafnode

    def __repr__(self):
        attrs = _repr_help_format_list(self.attributes)
        return ('DictNode(name=%r, type=%r, label=%r, default=%r)\n'
                '    attributes=[\n    %s    \n]\n'
                % (self.name, self.type, self.label, self.default, attrs))


class EnumerateNode(object):
    def __init__(self, name, select_multiple, default, options,
                 label=None, description=None, required=True):
        type_ = 'selectMultiple' if select_multiple else 'select'
        self.name = name
        self.type = type_
        self.label = label or name
        self.description = description or self.label
        self.required = required
        self.default = default
        self.options = options  # of type Option. These options are derived from the Type traits.

    def __repr__(self):
        return '%s(name=%r, type=%r, label=%r, default=%r, options=%r)' % (
            type(self).__name__, self.name, self.type, self.label, self.default, self.options)


class Option(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value

    def __repr__(self):
        return repr((self.name, self.value))




# -*- coding: utf-8 -*-
#
#
#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
# analysers necessary to run brain-simulations. You can use it stand alone or
# in conjunction with TheVirtualBrain-Framework Package. See content of the
# documentation-folder for more details. See also http://www.thevirtualbrain.org
#
# (c) 2012-2017, Baycrest Centre for Geriatric Care ("Baycrest") and others
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this
# program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#   CITATION:
# When using The Virtual Brain for scientific publications, please cite it as follows:
#
#   Paula Sanz Leon, Stuart A. Knock, M. Marmaduke Woodman, Lia Domide,
#   Jochen Mersmann, Anthony R. McIntosh, Viktor Jirsa (2013)
#       The Virtual Brain: a simulator of primate brain network dynamics.
#   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
#
#
"""
.. moduleauthor:: Lia Domide <lia.domide@codemart.ro>
"""

import numpy
import six
from tvb.basic.traits.core import TYPE_REGISTER

KEYWORD_PARAMS = "_parameters_"
KEYWORD_SEPARATOR = "_"
KEYWORD_OPTION = "option_"


def get_traited_subclasses(parent_class):
    """
    :param parent_class: SuperClass, to return valid sub-classes of this (e.g. Model).
    :return: {class_name: sub_class_instance}
        e.g. {'WilsonCowan': WilsonCowan, ....}
    """
    classes_list = TYPE_REGISTER.subclasses(parent_class)
    result = {}
    for class_instance in classes_list:
        result[class_instance.__name__] = class_instance
    return result


def get_traited_instance_for_name(class_name, parent_class, params_dictionary):
    """
    :param class_name: Short Traited Class name.
    :param parent_class: Traited basic type expected (e.g. Model)
    :param params_dictionary: dictionary of parameters to be passed to the constructor of the class.
    :return: Class instantiated corresponding to the given name (e.g. FHN() )
    """
    available_subclasses = get_traited_subclasses(parent_class)
    if class_name not in available_subclasses:
        return None
    class_instance = available_subclasses[class_name]
    entity = class_instance(**params_dictionary)
    return entity


def collapse_params(args, simple_select_list, parent=''):
    """ In case of variables with similar names:
    (name_parameters_[option_xx]_paramKey) collapse then into dictionary
    of parameters. This is used after parameters POST, on Operation Launch.
    """
    result = {}
    for name, value in args.items():
        short_name = name
        option = None
        key = None
        if name.find(KEYWORD_PARAMS) >= 0:
            short_name = name[0: (name.find(KEYWORD_PARAMS) + 11)]
            key = name[(name.find(KEYWORD_PARAMS) + 12):]
            if key.find(KEYWORD_OPTION) == 0:
                key = key[7:]  # Remove '_option_'
                option = key[0: key.find(KEYWORD_SEPARATOR)]
                key = key[key.find(KEYWORD_SEPARATOR) + 1:]

        if key is None:
            result[name] = value
        else:
            if short_name not in result:
                result[short_name] = {}
            if option is None:
                result[short_name][key] = value
            else:
                if option not in result[short_name]:
                    result[short_name][option] = {}
                result[short_name][option][key] = value

    for level1_name, level1_params in result.items():
        if KEYWORD_PARAMS[:-1] in level1_name and isinstance(level1_params, dict):
            short_parent_name = level1_name[0: level1_name.find(KEYWORD_PARAMS) - 10]
            if (parent + short_parent_name) in simple_select_list:
                # simple select
                if isinstance(result[short_parent_name], (str, unicode)):
                    parent_prefix = level1_name + KEYWORD_SEPARATOR + KEYWORD_OPTION
                    parent_prefix += result[short_parent_name]
                    parent_prefix += KEYWORD_SEPARATOR
                    # Ignore options in case of simple selects
                    # Take only attributes for current selected option.
                    if result[short_parent_name] in level1_params:
                        level1_params = level1_params[result[short_parent_name]]
                    else:
                        level1_params = {}
                else:
                    parent_prefix = level1_name

                transformed_params = collapse_params(level1_params, simple_select_list, parent + parent_prefix)
                result[level1_name] = transformed_params
            elif short_parent_name in result:
                # multiple select
                for level2_name, level2_params in level1_params.items():
                    parent_prefix = level1_name + KEYWORD_SEPARATOR + KEYWORD_OPTION
                    parent_prefix += level2_name + KEYWORD_SEPARATOR
                    transformed_params = collapse_params(level2_params, simple_select_list, parent + parent_prefix)
                    result[level1_name][level2_name] = transformed_params
    return result


def try_parse(val):
    if isinstance(val, dict):
        return {str(k): try_parse(v) for k, v in six.iteritems(val)}
    if isinstance(val, list):
        return val

    try:
        return int(val)
    except Exception:
        try:
            return float(val)
        except Exception:
            if val.find('[') == 0:
                try:
                    return numpy.array(val.replace('[', '').replace(']', '').split(','), dtype=numpy.float64)
                except Exception:
                    pass
            return str(val)
